@@ -510,6 +510,8 @@ static const char *File_ext(const char *filename)
       return "image/jpeg";
    } else if (!dStrAsciiCasecmp(e, "png")) {
       return "image/png";
+   } else if (!dStrAsciiCasecmp(e, "svg")) {
+      return "image/svg+xml";
    } else if (!dStrAsciiCasecmp(e, "html") ||
               !dStrAsciiCasecmp(e, "xhtml") ||
               !dStrAsciiCasecmp(e, "htm") ||
@@ -998,7 +1000,7 @@ static void File_serve_client(void *data, int f_write)
 /*
  * Serve the client queue.
  */
-static void File_serve_clients()
+static void File_serve_clients(void)
 {
    int i, f_read, f_write;
    ClientInfo *client;
