@@ -1,5 +1,6 @@
 /*
    Copyright (C) 2003  Ferdi Franceschini <ferdif@optusnet.com.au>
+   Copyright (C) 2025  Rodrigo Arias Mallo <rodarima@gmail.com>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -25,7 +26,9 @@
 #include <sys/stat.h>
 #include <sys/wait.h>
 #include <sys/socket.h>
+#include <netinet/in.h>
 #include <netinet/tcp.h>
+#include <arpa/inet.h>
 
 #include <unistd.h>
 #include "dpid_common.h"
@@ -47,7 +50,7 @@ char *SharedKey = NULL;
  * This avoids that dillo instances connect to a stale port after dpid
  * has exited (e.g. after a reboot).
  */
-void cleanup()
+void cleanup(void)
 {
    char *fname;
    fname = dStrconcat(dGethomedir(), "/", dotDILLO_DPID_COMM_KEYS, NULL);
@@ -112,7 +115,7 @@ static void terminator(int sig)
 
 /*! Establish handler for termination signals
  * and register cleanup with atexit */
-void est_dpi_terminator()
+void est_dpi_terminator(void)
 {
    struct sigaction act;
    sigset_t block;
@@ -344,7 +347,6 @@ int register_service(struct dp *dpi_attr, char *service)
  */
 int register_all(struct dp **attlist)
 {
-   DIR *user_dir_stream, *sys_dir_stream;
    char *user_dpidir = NULL, *sys_dpidir = NULL, *dpidrc = NULL;
    struct dirent *user_dirent, *sys_dirent;
    int st;
@@ -383,28 +385,43 @@ int register_all(struct dp **attlist)
 
    /* Get list of services in user's .dillo/dpi directory */
    snum = 0;
-   if (user_dpidir && (user_dir_stream = opendir(user_dpidir)) != NULL) {
-      while ((user_dirent = readdir(user_dir_stream)) != NULL) {
-         if (user_dirent->d_name[0] == '.')
-            continue;
-         *attlist = (struct dp *) dRealloc(*attlist, (snum + 1) * dp_sz);
-         st=get_dpi_attr(user_dpidir, user_dirent->d_name, &(*attlist)[snum]);
-         if (st == 0)
-            snum++;
+   if (user_dpidir) {
+      DIR *user_dir_stream = opendir(user_dpidir);
+      /* Only complain if error is other than not found as the user may not have
+       * any DPIs installed. */
+      if (user_dir_stream == NULL && errno != ENOENT) {
+         MSG_ERR("cannot open user dpi directory '%s': %s\n",
+               user_dpidir, strerror(errno));
+      } else {
+         while ((user_dirent = readdir(user_dir_stream)) != NULL) {
+            if (user_dirent->d_name[0] == '.')
+               continue;
+            *attlist = (struct dp *) dRealloc(*attlist, (snum + 1) * dp_sz);
+            st=get_dpi_attr(user_dpidir, user_dirent->d_name, &(*attlist)[snum]);
+            if (st == 0)
+               snum++;
+         }
+         closedir(user_dir_stream);
       }
-      closedir(user_dir_stream);
    }
-   if (sys_dpidir && (sys_dir_stream = opendir(sys_dpidir)) != NULL) {
-      /* if system service is not in user list then add it */
-      while ((sys_dirent = readdir(sys_dir_stream)) != NULL) {
-         if (sys_dirent->d_name[0] == '.')
-           continue;
-         *attlist = (struct dp *) dRealloc(*attlist, (snum + 1) * dp_sz);
-         st=get_dpi_attr(sys_dpidir, sys_dirent->d_name, &(*attlist)[snum]);
-         if (st == 0)
-            snum++;
+   if (sys_dpidir) {
+      DIR *sys_dir_stream = opendir(sys_dpidir);
+      /* For system DPIs always complain. */
+      if (sys_dir_stream == NULL) {
+         MSG_ERR("cannot open system dpi directory '%s': %s\n",
+               sys_dpidir, strerror(errno));
+      } else {
+         /* if system service is not in user list then add it */
+         while ((sys_dirent = readdir(sys_dir_stream)) != NULL) {
+            if (sys_dirent->d_name[0] == '.')
+              continue;
+            *attlist = (struct dp *) dRealloc(*attlist, (snum + 1) * dp_sz);
+            st=get_dpi_attr(sys_dpidir, sys_dirent->d_name, &(*attlist)[snum]);
+            if (st == 0)
+               snum++;
+         }
+         closedir(sys_dir_stream);
       }
-      closedir(sys_dir_stream);
    }
 
    dFree(sys_dpidir);
@@ -526,7 +543,7 @@ int fill_services_list(struct dp *attlist, int numdpis, Dlist **services_list)
  * Return a socket file descriptor
  * (useful to set socket options in a uniform way)
  */
-static int make_socket_fd()
+static int make_socket_fd(void)
 {
    int ret, one = 1;
 
@@ -564,7 +581,7 @@ int bind_socket_fd(int base_port, int *p_port)
 
    memset(&sin, 0, sizeof(sin));
    sin.sin_family = AF_INET;
-   sin.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
+   sin.sin_addr.s_addr = inet_addr("127.0.0.1");
 
    /* Try to bind a port on localhost */
    for (port = base_port; port <= last_port; ++port) {
@@ -618,7 +635,7 @@ int save_comm_keys(int srs_port)
  * \li Number of sockets (1 == success)
  * \li -1 on failure
  */
-int init_ids_srs_socket()
+int init_ids_srs_socket(void)
 {
    int srs_port, ret = -1;
 
@@ -754,7 +771,7 @@ void stop_active_dpis(struct dp *dpi_attr_list, int numdpis)
 
    memset(&sin, 0, sizeof(sin));
    sin.sin_family = AF_INET;
-   sin.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
+   sin.sin_addr.s_addr = inet_addr("127.0.0.1");
 
    for (i = 0; i < numdpis; i++) {
       /* Skip inactive dpis and filters */
@@ -806,7 +823,7 @@ void ignore_dpi_sockets(struct dp *dpi_attr_list, int numdpis)
  * \Return
  * Number of available dpis
  */
-int register_all_cmd()
+int register_all_cmd(void)
 {
    stop_active_dpis(dpi_attr_list, numdpis);
    free_plugin_list(&dpi_attr_list, numdpis);
