@@ -9,8 +9,8 @@
  * (at your option) any later version.
  */
 
+#include <errno.h>
 #include <stdio.h>
-#include <string.h>
 #include <unistd.h>
 #include "dpid_common.h"
 
@@ -24,17 +24,6 @@ void errmsg(char *caller, char *called, int errornum, char *file, int line)
       MSG_ERR("%s\n", dStrerror(errornum));
 }
 
-/*! Selector function for scandir
- * Do not scan files starting with '.'
- */
-int no_dotfiles(const struct dirent *filedat)
-{
-   if (filedat->d_name[0] == '.')
-      return 0;
-   else
-      return 1;
-}
-
 /*!
  * Provides an error checked write command.
  * Call this via the CKD_WRITE macro
@@ -52,3 +41,19 @@ ssize_t ckd_write(int fd, char *msg, char *file, int line)
    }
    return (ret);
 }
+
+/*!
+ * Provides an error checked close() call.
+ * Call this via the CKD_CLOSE macro
+ * \return close return value
+ */
+ssize_t ckd_close(int fd, char *file, int line)
+{
+   ssize_t ret;
+
+   ret = dClose(fd);
+   if (ret == -1)
+      MSG_ERR("%s:%d: close: %s\n", file, line, dStrerror(errno));
+   return ret;
+}
+
