@@ -19,6 +19,7 @@
 
 #include <stdio.h>
 
+#include "dlib/dlib.h"
 #include "../lout/msg.h"
 #include "../lout/debug.hh"
 #include "fltkcore.hh"
@@ -121,6 +122,7 @@ FltkFont::FltkFont (core::style::FontAttrs *attrs)
    int xx, xy, xw, xh;
    fl_text_extents("x", xx, xy, xw, xh);
    xHeight = xh;
+   zeroWidth = (int) fl_width("0");
    descent = fl_descent();
    ascent = fl_height() - descent;
 }
@@ -145,7 +147,7 @@ void FltkFont::initSystemFonts ()
    int k = Fl::set_fonts ("-*-iso10646-1");
    for (int i = 0; i < k; i++) {
       int t;
-      char *name = strdup (Fl::get_font_name ((Fl_Font) i, &t));
+      char *name = dStrdup (Fl::get_font_name ((Fl_Font) i, &t));
 
       // normalize font family names (strip off "bold", "italic")
       if (t & FL_ITALIC)
@@ -497,7 +499,7 @@ void FltkPlatform::detachView (core::View *view)
 {
    if (this->view != view)
       MSG_ERR("FltkPlatform::detachView: this->view: %p view: %p\n",
-              this->view, view);
+              (void *) this->view, (void *) view);
 
    for (container::typed::Iterator <ui::FltkResource> it =
            resources->iterator (); it.hasNext (); ) {
