@@ -2,6 +2,7 @@
  * Dillo Widget
  *
  * Copyright 2005-2007 Sebastian Geerken <sgeerken@dillo.org>
+ * Copyright 2024 Rodrigo Arias Mallo <rodarima@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -71,6 +72,12 @@ FltkViewport::FltkViewport (int X, int Y, int W, int H, const char *label):
    hasDragScroll = 1;
    scrollX = scrollY = scrollDX = scrollDY = 0;
    horScrolling = verScrolling = dragScrolling = 0;
+   scrollbarPageMode = false;
+   pageOverlap = 50;
+   pageScrolling = core::NONE_CMD;
+
+   pageScrollDelay = 0.300;
+   pageScrollInterval = 0.100;
 
    gadgetOrientation[0] = GADGET_HORIZONTAL;
    gadgetOrientation[1] = GADGET_HORIZONTAL;
@@ -116,18 +123,27 @@ void FltkViewport::adjustScrollbarsAndGadgetsAllocation ()
       vdiff = hscrollbar->visible () ? SCROLLBAR_THICKNESS : 0;
    }
 
-   hscrollbar->resize(x (), y () + h () - SCROLLBAR_THICKNESS,
-                      w () - hdiff, SCROLLBAR_THICKNESS);
-   vscrollbar->resize(x () + w () - SCROLLBAR_THICKNESS, y (),
-                      SCROLLBAR_THICKNESS, h () - vdiff);
+   if (scrollbarOnLeft) {
+      hscrollbar->resize(x () + hdiff, y () + h () - SCROLLBAR_THICKNESS,
+                         w () - hdiff, SCROLLBAR_THICKNESS);
+      vscrollbar->resize(x (), y (),
+                         SCROLLBAR_THICKNESS, h () - vdiff);
+   } else {
+      hscrollbar->resize(x (), y () + h () - SCROLLBAR_THICKNESS,
+                         w () - hdiff, SCROLLBAR_THICKNESS);
+      vscrollbar->resize(x () + w () - SCROLLBAR_THICKNESS, y (),
+                         SCROLLBAR_THICKNESS, h () - vdiff);
+   }
 
-   int X = x () + w () - SCROLLBAR_THICKNESS;
-   int Y = y () + h () - SCROLLBAR_THICKNESS;
+   //int X = x () + w () - SCROLLBAR_THICKNESS;
+   //int Y = y () + h () - SCROLLBAR_THICKNESS;
    for (Iterator <TypedPointer < Fl_Widget> > it = gadgets->iterator ();
         it.hasNext (); ) {
       Fl_Widget *widget = it.getNext()->getTypedValue ();
       widget->resize(x (), y (), SCROLLBAR_THICKNESS, SCROLLBAR_THICKNESS);
 
+      /* FIXME: This has no effect */
+#if 0
       switch (gadgetOrientation [visibility]) {
       case GADGET_VERTICAL:
          Y -= SCROLLBAR_THICKNESS;
@@ -137,7 +153,10 @@ void FltkViewport::adjustScrollbarsAndGadgetsAllocation ()
          X -= SCROLLBAR_THICKNESS;
          break;
       }
+#endif
    }
+
+   adjustScrollbarValues();
 }
 
 void FltkViewport::adjustScrollbarValues ()
@@ -219,17 +238,29 @@ void FltkViewport::draw ()
       draw_child (*hscrollbar);
    if (draw && vis_vs && vis_hs) {
       fl_color(FL_BACKGROUND_COLOR);
-      fl_rectf(x()+w()-vis_vs, y()+h()-vis_hs, vis_vs, vis_hs);
+      if (scrollbarOnLeft) {
+         fl_rectf(x(), y()+h()-vis_hs, vis_vs, vis_hs);
+      } else {
+         fl_rectf(x()+w()-vis_vs, y()+h()-vis_hs, vis_vs, vis_hs);
+      }
    }
    // main area
    if (d == FL_DAMAGE_CHILD && (draw_vs || draw_hs)) {
       _MSG("none\n");
    } else if (d == (FL_DAMAGE_SCROLL | FL_DAMAGE_CHILD)) {
-      fl_scroll(x(), y(), w() - vis_vs, h() - vis_hs,
+      int x = this->x();
+
+      if (scrollbarOnLeft)
+         x += vis_vs;
+      fl_scroll(x, y(), w() - vis_vs, h() - vis_hs,
                 -scrollDX, -scrollDY, draw_area, this);
       _MSG("fl_scroll()\n");
    } else {
-      draw_area(this, x(), y(), w() - vis_vs, h() - vis_hs);
+      int x = this->x();
+
+      if (scrollbarOnLeft)
+         x += vis_vs;
+      draw_area(this, x, y(), w() - vis_vs, h() - vis_hs);
       _MSG("draw_area()\n");
    }
 
@@ -268,8 +299,41 @@ int FltkViewport::handle (int event)
 
    case FL_PUSH:
       if (vscrollbar->visible() && Fl::event_inside(vscrollbar)) {
-         if (vscrollbar->handle(event))
+         if (scrollbarPageMode ^ (bool) Fl::event_shift()) {
+            /* Check top and bottom actions first */
+            int yclick = Fl::event_y();
+            int ytop = y() + SCROLLBAR_THICKNESS;
+            int ybottom = y() + h() - SCROLLBAR_THICKNESS;
+            if (hscrollbar->visible())
+               ybottom -= SCROLLBAR_THICKNESS;
+
+            if (yclick <= ytop) {
+               scroll(core::TOP_CMD);
+               return 1;
+            } else if (yclick >= ybottom) {
+               scroll(core::BOTTOM_CMD);
+               return 1;
+            }
+
+            if (Fl::event_button() == FL_LEFT_MOUSE)
+               pageScrolling = core::SCREEN_DOWN_CMD;
+            else if (Fl::event_button() == FL_RIGHT_MOUSE)
+               pageScrolling = core::SCREEN_UP_CMD;
+            else
+               pageScrolling = core::NONE_CMD;
+
+            if (pageScrolling != core::NONE_CMD) {
+               scroll(pageScrolling);
+               /* Repeat until released */
+               if (!Fl::has_timeout(repeatPageScroll, this))
+                  Fl::add_timeout(pageScrollDelay, repeatPageScroll, this);
+
+               return 1;
+            }
+         }
+         if (vscrollbar->handle(event)) {
             verScrolling = 1;
+         }
       } else if (hscrollbar->visible() && Fl::event_inside(hscrollbar)) {
          if (hscrollbar->handle(event))
             horScrolling = 1;
@@ -312,10 +376,21 @@ int FltkViewport::handle (int event)
       break;
 
    case FL_MOUSEWHEEL:
+      if ((vscrollbar->visible() && Fl::event_inside(vscrollbar)) ||
+            Fl::event_shift()) {
+         if (Fl::event_dy() > 0) {
+            scroll(core::SCREEN_DOWN_CMD);
+            return 1;
+         } else if (Fl::event_dy() < 0) {
+            scroll(core::SCREEN_UP_CMD);
+            return 1;
+         }
+      }
       return (Fl::event_dx() ? hscrollbar : vscrollbar)->handle(event);
       break;
 
    case FL_RELEASE:
+      Fl::remove_timeout(repeatPageScroll);
       Fl::remove_timeout(selectionScroll);
       if (Fl::event_button() == FL_MIDDLE_MOUSE) {
          setCursor (core::style::CURSOR_DEFAULT);
@@ -328,12 +403,27 @@ int FltkViewport::handle (int event)
       break;
 
    case FL_ENTER:
+      if (vscrollbar->visible() && Fl::event_inside(vscrollbar))
+         return vscrollbar->handle(event);
+      if (hscrollbar->visible() && Fl::event_inside(hscrollbar))
+         return hscrollbar->handle(event);
       /* could be the result of, e.g., closing another window. */
       mouse_x = Fl::event_x();
       mouse_y = Fl::event_y();
       positionChanged();
       break;
 
+   case FL_MOVE:
+      /* Use LEAVE in order not to be over a link, etc., anymore. */
+      if (vscrollbar->visible() && Fl::event_inside(vscrollbar)) {
+         (void)FltkWidgetView::handle(FL_LEAVE);
+         return vscrollbar->handle(event);
+      }
+      if (hscrollbar->visible() && Fl::event_inside(hscrollbar)) {
+         (void)FltkWidgetView::handle(FL_LEAVE);
+         return hscrollbar->handle(event);
+      }
+      break;
    case FL_LEAVE:
       mouse_x = mouse_y = -1;
       break;
@@ -373,6 +463,16 @@ void FltkViewport::setScrollStep(int step)
    hscrollbar->linesize(step);
 }
 
+void FltkViewport::setPageOverlap(int overlap)
+{
+   pageOverlap = overlap;
+}
+
+void FltkViewport::setScrollbarPageMode(bool enable)
+{
+   scrollbarPageMode = enable;
+}
+
 bool FltkViewport::usesViewport ()
 {
    return true;
@@ -426,22 +526,24 @@ void FltkViewport::scroll (int dx, int dy)
 
 void FltkViewport::scroll (core::ScrollCommand cmd)
 {
+   int hdiff = vscrollbar->visible () ? SCROLLBAR_THICKNESS : 0;
+   int vdiff = hscrollbar->visible () ? SCROLLBAR_THICKNESS : 0;
    if (cmd == core::SCREEN_UP_CMD) {
-      scroll (0, -h () + vscrollbar->linesize ());
+      scroll (0, -h () + pageOverlap + vdiff);
    } else if (cmd == core::SCREEN_DOWN_CMD) {
-      scroll (0, h () - vscrollbar->linesize ());
+      scroll (0, h () - pageOverlap - vdiff);
    } else if (cmd == core::SCREEN_LEFT_CMD) {
-      scroll (-w() + hscrollbar->linesize (), 0);
+      scroll (-w() + pageOverlap + hdiff, 0);
    } else if (cmd == core::SCREEN_RIGHT_CMD) {
-      scroll (w() - hscrollbar->linesize (), 0);
+      scroll (w() - pageOverlap - hdiff, 0);
    } else if (cmd == core::LINE_UP_CMD) {
-      scroll (0, (int) -vscrollbar->linesize ());
+      scroll (0, -vscrollbar->linesize ());
    } else if (cmd == core::LINE_DOWN_CMD) {
-      scroll (0, (int) vscrollbar->linesize ());
+      scroll (0, vscrollbar->linesize ());
    } else if (cmd == core::LEFT_CMD) {
-      scroll ((int) -hscrollbar->linesize (), 0);
+      scroll (-hscrollbar->linesize (), 0);
    } else if (cmd == core::RIGHT_CMD) {
-      scroll ((int) hscrollbar->linesize (), 0);
+      scroll (hscrollbar->linesize (), 0);
    } else if (cmd == core::TOP_CMD) {
       scrollTo (scrollX, 0);
    } else if (cmd == core::BOTTOM_CMD) {
@@ -475,6 +577,24 @@ void FltkViewport::selectionScroll (void *data)
    Fl::repeat_timeout(0.025, selectionScroll, data);
 }
 
+void FltkViewport::repeatPageScroll ()
+{
+   scroll(pageScrolling);
+   Fl::repeat_timeout(pageScrollInterval, repeatPageScroll, this);
+}
+
+void FltkViewport::repeatPageScroll (void *data)
+{
+   ((FltkViewport *)data)->repeatPageScroll ();
+}
+
+void FltkViewport::setScrollbarOnLeft (bool enable)
+{
+   scrollbarOnLeft = enable ? 1 : 0;
+   adjustScrollbarsAndGadgetsAllocation();
+   damage(FL_DAMAGE_ALL);
+}
+
 void FltkViewport::setViewportSize (int width, int height,
                                     int hScrollbarThickness,
                                     int vScrollbarThickness)
