@@ -21,6 +21,7 @@
 
 #include "hyphenator.hh"
 
+#include "dlib/dlib.h"
 #include "../lout/misc.hh"
 #include "../lout/unicode.hh"
 #include <limits.h>
@@ -224,17 +225,6 @@ bool Hyphenator::isHyphenationCandidate (const char *word)
  */
 bool Hyphenator::isCharPartOfActualWord (char *s)
 {
-#if 0
-   // Return true when "s" points to a letter.
-   return (s[0] >= 'a' && s[0] <= 'z') ||
-      // UTF-8: starts with 0xc3
-      ((unsigned char)s[0] == 0xc3 &&
-       ((unsigned char)s[1] == 0xa4 /* ä */ ||
-        (unsigned char)s[1] == 0xb6 /* ö */ ||
-        (unsigned char)s[1] == 0xbc /* ü */ ||
-        (unsigned char)s[1] == 0x9f /* ß */ ));
-#endif
-
    return isAlpha (decodeUtf8 (s));
 }
 
@@ -244,7 +234,7 @@ bool Hyphenator::isCharPartOfActualWord (char *s)
 int *Hyphenator::hyphenateWord(core::Platform *platform,
                                const char *word, int *numBreaks)
 {
-   if ((trie == NULL && exceptions ==NULL) || !isHyphenationCandidate (word)) {
+   if ((trie == NULL && exceptions == NULL) || !isHyphenationCandidate (word)) {
       *numBreaks = 0;
       return NULL;
    }
@@ -406,7 +396,7 @@ TrieBuilder::~TrieBuilder ()
 void TrieBuilder::insert (const char *key, const char *value)
 {
    dataList->increase ();
-   dataList->getLastRef ()->key = (unsigned char *) strdup(key);
+   dataList->getLastRef ()->key = (unsigned char *) dStrdup(key);
    dataList->getLastRef ()->value = dataZone->strdup (value);
 }
 
@@ -578,7 +568,7 @@ int Trie::load (FILE *file)
          continue;
 
       char data[LEN + 1];
-      int n = sscanf (s, "%u, %u, %s", &c, &next, data);
+      int n = sscanf (s, "%d, %d, %s", &c, &next, data);
 
       if (n >= 2 && c >= 0 && c < 256 && next >= 0) {
          tree.increase ();
