@@ -93,63 +93,79 @@ void SelectionState::resetLink ()
 bool SelectionState::buttonPress (Iterator *it, int charPos, int linkNo,
                                   EventButton *event)
 {
+   DBG_IF_RTFL {
+      misc::StringBuffer sb;
+      it->intoStringBuffer (&sb);
+      DBG_OBJ_ENTER ("events", 0, "buttonPress", "[%s], %d, %d, ...",
+                     sb.getChars (), charPos, linkNo);
+   }
+
    Widget *itWidget = it->getWidget ();
    bool ret = false;
 
-   if (!event) return ret;
-
-   if (event->button == 3) {
-      // menu popup
-      layout->emitLinkPress (itWidget, linkNo, -1, -1, -1, event);
-      ret = true;
-   } else if (linkNo != -1) {
-      // link handling
-      (void) layout->emitLinkPress (itWidget, linkNo, -1, -1, -1, event);
-      resetLink ();
-      linkState = LINK_PRESSED;
-      linkButton = event->button;
-      DeepIterator *newLink = new DeepIterator (it);
-      if (newLink->isEmpty ()) {
-         delete newLink;
+   if (event) {
+      if (event->button == 3) {
+         // menu popup
+         layout->emitLinkPress (itWidget, linkNo, -1, -1, -1, event);
+         ret = true;
+      } else if (linkNo != -1) {
+         // link handling
+         (void) layout->emitLinkPress (itWidget, linkNo, -1, -1, -1, event);
          resetLink ();
-      } else {
-         link = newLink;
-         // It may be that the user has pressed on something activatable
-         // (linkNo != -1), but there is no contents, e.g. with images
-         // without ALTernative text.
-         if (link) {
-            linkChar = correctCharPos (link, charPos);
-            linkNumber = linkNo;
+         linkState = LINK_PRESSED;
+         linkButton = event->button;
+         DeepIterator *newLink = new DeepIterator (it);
+         if (newLink->isEmpty ()) {
+            delete newLink;
+            resetLink ();
+         } else {
+            link = newLink;
+            // It may be that the user has pressed on something activatable
+            // (linkNo != -1), but there is no contents, e.g. with images
+            // without ALTernative text.
+            if (link) {
+               linkChar = correctCharPos (link, charPos);
+               linkNumber = linkNo;
+            }
          }
-      }
-      // We do not return the value of the signal method,
-      // but we do actually process this event.
-      ret = true;
-   } else if (event->button == 1) {
-      // normal selection handling
-      highlight (false, 0);
-      resetSelection ();
-
-      selectionState = SELECTING;
-      DeepIterator *newFrom = new DeepIterator (it);
-      if (newFrom->isEmpty ()) {
-         delete newFrom;
+         // We do not return the value of the signal method,
+         // but we do actually process this event.
+         ret = true;
+      } else if (event->button == 1) {
+         // normal selection handling
+         highlight (false, 0);
          resetSelection ();
-      } else {
-         from = newFrom;
-         fromChar = correctCharPos (from, charPos);
-         to = from->cloneDeepIterator ();
-         toChar = correctCharPos (to, charPos);
+
+         selectionState = SELECTING;
+         DeepIterator *newFrom = new DeepIterator (it);
+         if (newFrom->isEmpty ()) {
+            delete newFrom;
+            resetSelection ();
+         } else {
+            from = newFrom;
+            fromChar = correctCharPos (from, charPos);
+            to = from->cloneDeepIterator ();
+            toChar = correctCharPos (to, charPos);
+         }
+         ret = true;
       }
-      ret = true;
    }
 
+   DBG_OBJ_MSGF ("events", 1, "=> %s", ret ? "true" : "false");
+   DBG_OBJ_LEAVE ();
    return ret;
 }
 
 bool SelectionState::buttonRelease (Iterator *it, int charPos, int linkNo,
                                     EventButton *event)
 {
+   DBG_IF_RTFL {
+      misc::StringBuffer sb;
+      it->intoStringBuffer (&sb);
+      DBG_OBJ_ENTER ("events", 0, "buttonRelease", "[%s], %d, %d, ...",
+                     sb.getChars (), charPos, linkNo);
+   }
+
    Widget *itWidget = it->getWidget ();
    bool ret = false;
 
@@ -186,6 +202,8 @@ bool SelectionState::buttonRelease (Iterator *it, int charPos, int linkNo,
       }
    }
 
+   DBG_OBJ_MSGF ("events", 1, "=> %s", ret ? "true" : "false");
+   DBG_OBJ_LEAVE ();
    return ret;
 }
 
