@@ -2,6 +2,7 @@
  * File: mime.c
  *
  * Copyright (C) 2000-2007 Jorge Arellano Cid <jcid@dillo.org>
+ * Copyright (C) 2024 Rodrigo Arias Mallo <rodarima@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -29,8 +30,8 @@ static int MimeMajItemsSize = 0, MimeMajItemsMax = 8;
 static MimeItem_t *MimeMajItems = NULL;
 
 
-/*
- * Add a specific MIME type (as "image/png") to our viewer list
+/**
+ * Add a specific MIME type (as "image/png") to our viewer list.
  * 'Key' is the content-type string that identifies the MIME type
  * 'Method' is the function that handles it
  */
@@ -43,8 +44,8 @@ static int Mime_add_minor_type(const char *Key, Viewer_t Method)
    return 0;
 }
 
-/*
- * Add a major MIME type (as "text") to our viewer list
+/**
+ * Add a major MIME type (as "text") to our viewer list.
  * 'Key' is the content-type string that identifies the MIME type
  * 'Method' is the function that handles it
  */
@@ -57,8 +58,8 @@ static int Mime_add_major_type(const char *Key, Viewer_t Method)
    return 0;
 }
 
-/*
- * Search the list of specific MIME viewers, for a Method that matches 'Key'
+/**
+ * Search the list of specific MIME viewers for a Method that matches 'Key'.
  * 'Key' is the content-type string that identifies the MIME type
  */
 static Viewer_t Mime_minor_type_fetch(const char *Key, uint_t Size)
@@ -73,8 +74,8 @@ static Viewer_t Mime_minor_type_fetch(const char *Key, uint_t Size)
    return NULL;
 }
 
-/*
- * Search the list of major MIME viewers, for a Method that matches 'Key'
+/**
+ * Search the list of major MIME viewers for a Method that matches 'Key'.
  * 'Key' is the content-type string that identifies the MIME type
  */
 static Viewer_t Mime_major_type_fetch(const char *Key, uint_t Size)
@@ -90,10 +91,10 @@ static Viewer_t Mime_major_type_fetch(const char *Key, uint_t Size)
 }
 
 
-/*
+/**
  * Initializes Mime module and, sets the supported Mime types.
  */
-void a_Mime_init()
+void a_Mime_init(void)
 {
 #ifdef ENABLE_GIF
    Mime_add_minor_type("image/gif", a_Dicache_gif_image);
@@ -106,16 +107,23 @@ void a_Mime_init()
 #ifdef ENABLE_PNG
    Mime_add_minor_type("image/png", a_Dicache_png_image);
    Mime_add_minor_type("image/x-png", a_Dicache_png_image);    /* deprecated */
+#endif
+#ifdef ENABLE_WEBP
+   Mime_add_minor_type("image/webp", a_Dicache_webp_image);
+#endif
+#ifdef ENABLE_SVG
+   Mime_add_minor_type("image/svg+xml", a_Dicache_svg_image);
 #endif
    Mime_add_minor_type("text/html", a_Html_text);
    Mime_add_minor_type("application/xhtml+xml", a_Html_text);
+   Mime_add_minor_type("application/json", a_Plain_text);
 
    /* Add a major type to handle all the text stuff */
    Mime_add_major_type("text", a_Plain_text);
 }
 
 
-/*
+/**
  * Get the handler for the MIME type.
  *
  * Return Value:
