@@ -2,6 +2,7 @@
  * File: cache.c
  *
  * Copyright 2000-2007 Jorge Arellano Cid <jcid@dillo.org>
+ * Copyright 2024 Rodrigo Arias Mallo <rodarima@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -10,6 +11,7 @@
  */
 
 /*
+ * @file
  * Dillo's cache module
  */
 
@@ -26,6 +28,7 @@
 #include "dicache.h"
 #include "nav.h"
 #include "cookies.h"
+#include "hsts.h"
 #include "misc.h"
 #include "capi.h"
 #include "decode.h"
@@ -34,9 +37,9 @@
 #include "timeout.hh"
 #include "uicmd.hh"
 
-/* Maximum initial size for the automatically-growing data buffer */
+/** Maximum initial size for the automatically-growing data buffer */
 #define MAX_INIT_BUF  1024*1024
-/* Maximum filesize for a URL, before offering a download */
+/** Maximum filesize for a URL, before offering a download */
 #define HUGE_FILESIZE 15*1024*1024
 
 /*
@@ -44,37 +47,37 @@
  */
 
 typedef struct {
-   const DilloUrl *Url;      /* Cached Url. Url is used as a primary Key */
-   char *TypeDet;            /* MIME type string (detected from data) */
-   char *TypeHdr;            /* MIME type string as from the HTTP Header */
-   char *TypeMeta;           /* MIME type string from META HTTP-EQUIV */
-   char *TypeNorm;           /* MIME type string normalized */
-   Dstr *Header;             /* HTTP header */
-   const DilloUrl *Location; /* New URI for redirects */
-   Dlist *Auth;              /* Authentication fields */
-   Dstr *Data;               /* Pointer to raw data */
-   Dstr *UTF8Data;           /* Data after charset translation */
-   int DataRefcount;         /* Reference count */
-   Decode *TransferDecoder;  /* Transfer decoder (e.g., chunked) */
-   Decode *ContentDecoder;   /* Data decoder (e.g., gzip) */
-   Decode *CharsetDecoder;   /* Translates text to UTF-8 encoding */
-   int ExpectedSize;         /* Goal size of the HTTP transfer (0 if unknown)*/
-   int TransferSize;         /* Actual length of the HTTP transfer */
-   uint_t Flags;             /* See Flag Defines in cache.h */
+   const DilloUrl *Url;      /**< Cached Url. Url is used as a primary Key */
+   char *TypeDet;            /**< MIME type string (detected from data) */
+   char *TypeHdr;            /**< MIME type string as from the HTTP Header */
+   char *TypeMeta;           /**< MIME type string from META HTTP-EQUIV */
+   char *TypeNorm;           /**< MIME type string normalized */
+   Dstr *Header;             /**< HTTP header */
+   const DilloUrl *Location; /**< New URI for redirects */
+   Dlist *Auth;              /**< Authentication fields */
+   Dstr *Data;               /**< Pointer to raw data */
+   Dstr *UTF8Data;           /**< Data after charset translation */
+   int DataRefcount;         /**< Reference count */
+   DecodeTransfer *TransferDecoder;  /**< Transfer decoder (e.g., chunked) */
+   Decode *ContentDecoder;   /**< Data decoder (e.g., gzip) */
+   Decode *CharsetDecoder;   /**< Translates text to UTF-8 encoding */
+   int ExpectedSize;         /**< Goal size of the HTTP transfer (0 if unknown)*/
+   int TransferSize;         /**< Actual length of the HTTP transfer */
+   uint_t Flags;             /**< See Flag Defines in cache.h */
 } CacheEntry_t;
 
 
 /*
  *  Local data
  */
-/* A sorted list for cached data. Holds pointers to CacheEntry_t structs */
+/** A sorted list for cached data. Holds pointers to CacheEntry_t structs */
 static Dlist *CachedURLs;
 
-/* A list for cache clients.
+/** A list for cache clients.
  * Although implemented as a list, we'll call it ClientQueue  --Jcid */
 static Dlist *ClientQueue;
 
-/* A list for delayed clients (it holds weak pointers to cache entries,
+/** A list for delayed clients (it holds weak pointers to cache entries,
  * which are used to make deferred calls to Cache_process_queue) */
 static Dlist *DelayedQueue;
 static uint_t DelayedQueueIdleId = 0;
@@ -88,7 +91,7 @@ static void Cache_delayed_process_queue(CacheEntry_t *entry);
 static void Cache_auth_entry(CacheEntry_t *entry, BrowserWindow *bw);
 static void Cache_entry_inject(const DilloUrl *Url, Dstr *data_ds);
 
-/*
+/**
  * Determine if two cache entries are equal (used by CachedURLs)
  */
 static int Cache_entry_cmp(const void *v1, const void *v2)
@@ -98,7 +101,7 @@ static int Cache_entry_cmp(const void *v1, const void *v2)
    return a_Url_cmp(d1->Url, d2->Url);
 }
 
-/*
+/**
  * Determine if two cache entries are equal, using a URL as key.
  */
 static int Cache_entry_by_url_cmp(const void *v1, const void *v2)
@@ -109,7 +112,7 @@ static int Cache_entry_by_url_cmp(const void *v1, const void *v2)
    return a_Url_cmp(u1, u2);
 }
 
-/*
+/**
  * Initialize cache data
  */
 void a_Cache_init(void)
@@ -130,7 +133,7 @@ void a_Cache_init(void)
 
 /* Client operations ------------------------------------------------------ */
 
-/*
+/**
  * Add a client to ClientQueue.
  *  - Every client-field is just a reference (except 'Web').
  *  - Return a unique number for identifying the client.
@@ -161,7 +164,7 @@ static int Cache_client_enqueue(const DilloUrl *Url, DilloWeb *Web,
    return ClientKey;
 }
 
-/*
+/**
  * Compare function for searching a Client by its key
  */
 static int Cache_client_by_key_cmp(const void *client, const void *key)
@@ -169,7 +172,7 @@ static int Cache_client_by_key_cmp(const void *client, const void *key)
    return ((CacheClient_t *)client)->Key - VOIDP2INT(key);
 }
 
-/*
+/**
  * Remove a client from the queue
  */
 static void Cache_client_dequeue(CacheClient_t *Client)
@@ -184,7 +187,7 @@ static void Cache_client_dequeue(CacheClient_t *Client)
 
 /* Entry operations ------------------------------------------------------- */
 
-/*
+/**
  * Set safe values for a new cache entry
  */
 static void Cache_entry_init(CacheEntry_t *NewEntry, const DilloUrl *Url)
@@ -205,10 +208,10 @@ static void Cache_entry_init(CacheEntry_t *NewEntry, const DilloUrl *Url)
    NewEntry->CharsetDecoder = NULL;
    NewEntry->ExpectedSize = 0;
    NewEntry->TransferSize = 0;
-   NewEntry->Flags = CA_IsEmpty;
+   NewEntry->Flags = CA_IsEmpty | CA_InProgress | CA_KeepAlive;
 }
 
-/*
+/**
  * Get the data structure for a cached URL (using 'Url' as the search key)
  * If 'Url' isn't cached, return NULL
  */
@@ -217,7 +220,7 @@ static CacheEntry_t *Cache_entry_search(const DilloUrl *Url)
    return dList_find_sorted(CachedURLs, Url, Cache_entry_by_url_cmp);
 }
 
-/*
+/**
  * Given a URL, find its cache entry, following redirections.
  */
 static CacheEntry_t *Cache_entry_search_with_redirect(const DilloUrl *Url)
@@ -241,7 +244,7 @@ static CacheEntry_t *Cache_entry_search_with_redirect(const DilloUrl *Url)
    return entry;
 }
 
-/*
+/**
  * Allocate and set a new entry in the cache list
  */
 static CacheEntry_t *Cache_entry_add(const DilloUrl *Url)
@@ -259,7 +262,7 @@ static CacheEntry_t *Cache_entry_add(const DilloUrl *Url)
    return new_entry;
 }
 
-/*
+/**
  * Inject full page content directly into the cache.
  * Used for "about:splash". May be used for "about:cache" too.
  */
@@ -269,7 +272,7 @@ static void Cache_entry_inject(const DilloUrl *Url, Dstr *data_ds)
 
    if (!(entry = Cache_entry_search(Url)))
       entry = Cache_entry_add(Url);
-   entry->Flags |= CA_GotData + CA_GotHeader + CA_GotLength + CA_InternalUrl;
+   entry->Flags = CA_GotHeader + CA_GotLength + CA_InternalUrl;
    if (data_ds->len)
       entry->Flags &= ~CA_IsEmpty;
    dStr_truncate(entry->Data, 0);
@@ -278,7 +281,7 @@ static void Cache_entry_inject(const DilloUrl *Url, Dstr *data_ds)
    entry->ExpectedSize = entry->TransferSize = entry->Data->len;
 }
 
-/*
+/**
  *  Free Authentication fields.
  */
 static void Cache_auth_free(Dlist *auth)
@@ -290,7 +293,7 @@ static void Cache_auth_free(Dlist *auth)
    dList_free(auth);
 }
 
-/*
+/**
  *  Free the components of a CacheEntry_t struct.
  */
 static void Cache_entry_free(CacheEntry_t *entry)
@@ -308,13 +311,13 @@ static void Cache_entry_free(CacheEntry_t *entry)
    if (entry->CharsetDecoder)
       a_Decode_free(entry->CharsetDecoder);
    if (entry->TransferDecoder)
-      a_Decode_free(entry->TransferDecoder);
+      a_Decode_transfer_free(entry->TransferDecoder);
    if (entry->ContentDecoder)
       a_Decode_free(entry->ContentDecoder);
    dFree(entry);
 }
 
-/*
+/**
  * Remove an entry, from the cache.
  * All the entry clients are removed too! (it may stop rendering of this
  * same resource on other windows, but nothing more).
@@ -348,7 +351,7 @@ static void Cache_entry_remove(CacheEntry_t *entry, DilloUrl *url)
    Cache_entry_free(entry);
 }
 
-/*
+/**
  * Wrapper for capi.
  */
 void a_Cache_entry_remove_by_url(DilloUrl *url)
@@ -358,7 +361,7 @@ void a_Cache_entry_remove_by_url(DilloUrl *url)
 
 /* Misc. operations ------------------------------------------------------- */
 
-/*
+/**
  * Try finding the url in the cache. If it hits, send the cache contents
  * from there. If it misses, set up a new connection.
  *
@@ -368,7 +371,7 @@ void a_Cache_entry_remove_by_url(DilloUrl *url)
  *   Note: 'Call' and/or 'CbData' can be NULL, in that case they get set
  *   later by a_Web_dispatch_by_type, based on content/type and 'Web' data.
  *
- * Return value: A primary key for identifying the client,
+ * @return A primary key for identifying the client,
  */
 int a_Cache_open_url(void *web, CA_Callback_t Call, void *CbData)
 {
@@ -397,7 +400,7 @@ int a_Cache_open_url(void *web, CA_Callback_t Call, void *CbData)
    return ClientKey;
 }
 
-/*
+/**
  * Get cache entry status
  */
 uint_t a_Cache_get_flags(const DilloUrl *url)
@@ -406,7 +409,7 @@ uint_t a_Cache_get_flags(const DilloUrl *url)
    return (entry ? entry->Flags : 0);
 }
 
-/*
+/**
  * Get cache entry status (following redirections).
  */
 uint_t a_Cache_get_flags_with_redirection(const DilloUrl *url)
@@ -415,7 +418,7 @@ uint_t a_Cache_get_flags_with_redirection(const DilloUrl *url)
    return (entry ? entry->Flags : 0);
 }
 
-/*
+/**
  * Reference the cache data.
  */
 static void Cache_ref_data(CacheEntry_t *entry)
@@ -433,7 +436,7 @@ static void Cache_ref_data(CacheEntry_t *entry)
    }
 }
 
-/*
+/**
  * Unreference the cache data.
  */
 static void Cache_unref_data(CacheEntry_t *entry)
@@ -454,7 +457,7 @@ static void Cache_unref_data(CacheEntry_t *entry)
    }
 }
 
-/*
+/**
  * Get current content type.
  */
 static const char *Cache_current_content_type(CacheEntry_t *entry)
@@ -463,7 +466,7 @@ static const char *Cache_current_content_type(CacheEntry_t *entry)
           : entry->TypeHdr ? entry->TypeHdr : entry->TypeDet;
 }
 
-/*
+/**
  * Get current Content-Type for cache entry found by URL.
  */
 const char *a_Cache_get_content_type(const DilloUrl *url)
@@ -473,7 +476,7 @@ const char *a_Cache_get_content_type(const DilloUrl *url)
    return (entry) ? Cache_current_content_type(entry) : NULL;
 }
 
-/*
+/**
  * Get pointer to entry's data.
  */
 static Dstr *Cache_data(CacheEntry_t *entry)
@@ -481,10 +484,10 @@ static Dstr *Cache_data(CacheEntry_t *entry)
    return entry->UTF8Data ? entry->UTF8Data : entry->Data;
 }
 
-/*
+/**
  * Change Content-Type for cache entry found by url.
  * from = { "http" | "meta" }
- * Return new content type.
+ * @return new content type.
  */
 const char *a_Cache_set_content_type(const DilloUrl *url, const char *ctype,
                                      const char *from)
@@ -498,7 +501,7 @@ const char *a_Cache_set_content_type(const DilloUrl *url, const char *ctype,
    _MSG("a_Cache_set_content_type {%s} {%s}\n", ctype, URL_STR(url));
 
    curr = Cache_current_content_type(entry);
-   if  (entry->TypeMeta || (*from == 'h' && entry->TypeHdr) ) {
+   if (entry->TypeMeta || (*from == 'h' && entry->TypeHdr) ) {
       /* Type is already been set. Do nothing.
        * BTW, META overrides TypeHdr */
    } else {
@@ -519,7 +522,12 @@ const char *a_Cache_set_content_type(const DilloUrl *url, const char *ctype,
          } else if (*from == 'm' &&
                     !dStrnAsciiCasecmp(ctype, "text/xhtml", 10)) {
             /* WORKAROUND: doxygen uses "text/xhtml" in META */
-            entry->TypeNorm = dStrdup(entry->TypeDet);
+            if (charset) {
+               entry->TypeNorm = dStrconcat("application/xhtml+xml",
+                        "; charset=", charset, NULL);
+            } else {
+               entry->TypeNorm = dStrdup("application/xhtml+xml");
+            }
          }
          if (charset) {
             if (entry->CharsetDecoder)
@@ -537,9 +545,9 @@ const char *a_Cache_set_content_type(const DilloUrl *url, const char *ctype,
    return curr;
 }
 
-/*
+/**
  * Get the pointer to the URL document, and its size, from the cache entry.
- * Return: 1 cached, 0 not cached.
+ * @return 1 cached, 0 not cached.
  */
 int a_Cache_get_buf(const DilloUrl *Url, char **PBuf, int *BufSize)
 {
@@ -557,7 +565,7 @@ int a_Cache_get_buf(const DilloUrl *Url, char **PBuf, int *BufSize)
    return (entry ? 1 : 0);
 }
 
-/*
+/**
  * Unreference the data buffer when no longer using it.
  */
 void a_Cache_unref_buf(const DilloUrl *Url)
@@ -566,10 +574,10 @@ void a_Cache_unref_buf(const DilloUrl *Url)
 }
 
 
-/*
+/**
  * Extract a single field from the header, allocating and storing the value
  * in 'field'. ('fieldname' must not include the trailing ':')
- * Return a new string with the field-content if found (NULL on error)
+ * @return a new string with the field-content if found (NULL on error)
  * (This function expects a '\r'-stripped header, with one-line header fields)
  */
 static char *Cache_parse_field(const char *header, const char *fieldname)
@@ -603,7 +611,7 @@ static char *Cache_parse_field(const char *header, const char *fieldname)
    return NULL;
 }
 
-/*
+/**
  * Extract multiple fields from the header.
  */
 static Dlist *Cache_parse_multiple_fields(const char *header,
@@ -645,14 +653,15 @@ static Dlist *Cache_parse_multiple_fields(const char *header,
    return fields;
 }
 
-/*
+/**
  * Scan, allocate, and set things according to header info.
  * (This function needs the whole header to work)
  */
 static void Cache_parse_header(CacheEntry_t *entry)
 {
    char *header = entry->Header->str;
-   char *Length, *Type, *location_str, *encoding;
+   bool_t server1point0 = !strncmp(entry->Header->str, "HTTP/1.0", 8);
+   char *Length, *Type, *location_str, *encoding, *connection, *hsts;
 #ifndef DISABLE_COOKIES
    Dlist *Cookies;
 #endif
@@ -709,6 +718,25 @@ static void Cache_parse_header(CacheEntry_t *entry)
       dList_free(warnings);
    }
 
+   if (server1point0)
+      entry->Flags &= ~CA_KeepAlive;
+
+   if ((connection = Cache_parse_field(header, "Connection"))) {
+      if (!dStrAsciiCasecmp(connection, "close"))
+         entry->Flags &= ~CA_KeepAlive;
+      else if (server1point0 && !dStrAsciiCasecmp(connection, "keep-alive"))
+         entry->Flags |= CA_KeepAlive;
+      dFree(connection);
+   }
+
+   if (prefs.http_strict_transport_security &&
+       !dStrAsciiCasecmp(URL_SCHEME(entry->Url), "https") &&
+       a_Url_host_type(URL_HOST(entry->Url)) == URL_HOST_NAME &&
+       (hsts = Cache_parse_field(header, "Strict-Transport-Security"))) {
+      a_Hsts_set(hsts, entry->Url);
+      dFree(hsts);
+   }
+
    /*
     * Get Transfer-Encoding and initialize decoder
     */
@@ -789,7 +817,7 @@ static void Cache_parse_header(CacheEntry_t *entry)
    Cache_ref_data(entry);
 }
 
-/*
+/**
  * Consume bytes until the whole header is got (up to a "\r\n\r\n" sequence)
  * (Also unfold multi-line fields and strip '\r' chars from header)
  */
@@ -824,7 +852,41 @@ static int Cache_get_header(CacheEntry_t *entry,
    return 0;
 }
 
-/*
+static void Cache_finish_msg(CacheEntry_t *entry)
+{
+   if (!(entry->Flags & CA_InProgress)) {
+      /* already finished */
+      return;
+   }
+
+   if ((entry->ExpectedSize || entry->TransferSize) &&
+       entry->TypeHdr == NULL) {
+      MSG_HTTP("Message with a body lacked Content-Type header.\n");
+   }
+   if ((entry->Flags & CA_GotLength) &&
+       (entry->ExpectedSize != entry->TransferSize)) {
+      MSG_HTTP("Content-Length (%d) does NOT match message body (%d) for %s\n",
+               entry->ExpectedSize, entry->TransferSize, URL_STR_(entry->Url));
+   }
+   entry->Flags &= ~CA_InProgress;
+   if (entry->TransferDecoder) {
+      a_Decode_transfer_free(entry->TransferDecoder);
+      entry->TransferDecoder = NULL;
+   }
+   if (entry->ContentDecoder) {
+      a_Decode_free(entry->ContentDecoder);
+      entry->ContentDecoder = NULL;
+   }
+   dStr_fit(entry->Data);                /* fit buffer size! */
+
+   if ((entry = Cache_process_queue(entry))) {
+      if (entry->Flags & CA_GotHeader) {
+         Cache_unref_data(entry);
+      }
+   }
+}
+
+/**
  * Receive new data, update the reception buffer (for next read), update the
  * cache, and service the client queue.
  *
@@ -832,16 +894,17 @@ static int Cache_get_header(CacheEntry_t *entry,
  *  'Op' is the operation to perform
  *  'VPtr' is a (void) pointer to the IO control structure
  */
-void a_Cache_process_dbuf(int Op, const char *buf, size_t buf_size,
-                          const DilloUrl *Url)
+bool_t a_Cache_process_dbuf(int Op, const char *buf, size_t buf_size,
+                            const DilloUrl *Url)
 {
    int offset, len;
    const char *str;
    Dstr *dstr1, *dstr2, *dstr3;
+   bool_t done = FALSE;
    CacheEntry_t *entry = Cache_entry_search(Url);
 
    /* Assert a valid entry (not aborted) */
-   dReturn_if_fail (entry != NULL);
+   dReturn_val_if_fail (entry != NULL, FALSE);
 
    _MSG("__a_Cache_process_dbuf__\n");
 
@@ -865,7 +928,8 @@ void a_Cache_process_dbuf(int Op, const char *buf, size_t buf_size,
 
          /* Decode arrived data (<= 3 stages) */
          if (entry->TransferDecoder) {
-            dstr1 = a_Decode_process(entry->TransferDecoder, str, len);
+            dstr1 = a_Decode_transfer_process(entry->TransferDecoder, str,len);
+            done = a_Decode_transfer_finished(entry->TransferDecoder);
             str = dstr1->str;
             len = dstr1->len;
          }
@@ -886,54 +950,46 @@ void a_Cache_process_dbuf(int Op, const char *buf, size_t buf_size,
          if (entry->Data->len)
             entry->Flags &= ~CA_IsEmpty;
 
+         if ((entry->Flags & CA_GotLength) &&
+             (entry->TransferSize >= entry->ExpectedSize)) {
+            done = TRUE;
+         }
+         if (!(entry->Flags & CA_KeepAlive)) {
+            /* Let IOClose finish it later */
+            done = FALSE;
+         }
+
          entry = Cache_process_queue(entry);
+
+         if (entry && done)
+            Cache_finish_msg(entry);
       }
    } else if (Op == IOClose) {
-      if ((entry->ExpectedSize || entry->TransferSize) &&
-          entry->TypeHdr == NULL) {
-         MSG_HTTP("Message with a body lacked Content-Type header.\n");
-      }
-      if ((entry->Flags & CA_GotLength) &&
-          (entry->ExpectedSize != entry->TransferSize)) {
-         MSG_HTTP("Content-Length does NOT match message body at\n"
-                  "%s\n", URL_STR_(entry->Url));
-         MSG("Expected size: %d, Transfer size: %d\n",
-             entry->ExpectedSize, entry->TransferSize);
-      }
-      if (!entry->TransferSize && !(entry->Flags & CA_Redirect) &&
-          (entry->Flags & WEB_RootUrl)) {
-         char *eol = strchr(entry->Header->str, '\n');
-         if (eol) {
-            char *status_line = dStrndup(entry->Header->str,
-                                         eol - entry->Header->str);
-            MSG_HTTP("Body was empty. Server sent status: %s\n", status_line);
-            dFree(status_line);
-         }
-      }
-      entry->Flags |= CA_GotData;
-      entry->Flags &= ~CA_Stopped;          /* it may catch up! */
-      if (entry->TransferDecoder) {
-         a_Decode_free(entry->TransferDecoder);
-         entry->TransferDecoder = NULL;
-      }
-      if (entry->ContentDecoder) {
-         a_Decode_free(entry->ContentDecoder);
-         entry->ContentDecoder = NULL;
-      }
-      dStr_fit(entry->Data);                /* fit buffer size! */
-
-      if ((entry = Cache_process_queue(entry))) {
-         if (entry->Flags & CA_GotHeader) {
-            Cache_unref_data(entry);
-         }
-      }
+      Cache_finish_msg(entry);
    } else if (Op == IOAbort) {
-      /* unused */
-      MSG("a_Cache_process_dbuf Op = IOAbort; not implemented!\n");
+      entry->Flags |= CA_Aborted;
+      if (entry->Data->len) {
+         MSG("Premature close for %s\n", URL_STR(entry->Url));
+         Cache_finish_msg(entry);
+      } else {
+         int i;
+         CacheClient_t *Client;
+
+         for (i = 0; (Client = dList_nth_data(ClientQueue, i)); ++i) {
+            if (Client->Url == entry->Url) {
+               DilloWeb *web = (DilloWeb *)Client->Web;
+
+               a_Bw_remove_client(web->bw, Client->Key);
+               Cache_client_dequeue(Client);
+               --i; /* Keep the index value in the next iteration */
+            }
+         }
+      }
    }
+   return done;
 }
 
-/*
+/**
  * Process redirections (HTTP 30x answers)
  * (This is a work in progress --not finished yet)
  */
@@ -992,7 +1048,7 @@ typedef struct {
    BrowserWindow *bw;
 } CacheAuthData_t;
 
-/*
+/**
  * Ask for user/password and reload the page.
  */
 static void Cache_auth_callback(void *vdata)
@@ -1007,7 +1063,7 @@ static void Cache_auth_callback(void *vdata)
    a_Timeout_remove();
 }
 
-/*
+/**
  * Set a timeout function to ask for user/password.
  */
 static void Cache_auth_entry(CacheEntry_t *entry, BrowserWindow *bw)
@@ -1030,9 +1086,9 @@ static void Cache_auth_entry(CacheEntry_t *entry, BrowserWindow *bw)
    }
 }
 
-/*
+/**
  * Check whether a URL scheme is downloadable.
- * Return: 1 enabled, 0 disabled.
+ * @return 1 enabled, 0 disabled.
  */
 int a_Cache_download_enabled(const DilloUrl *url)
 {
@@ -1043,7 +1099,7 @@ int a_Cache_download_enabled(const DilloUrl *url)
    return 0;
 }
 
-/*
+/**
  * Don't process data any further, but let the cache fill the entry.
  * (Currently used to handle WEB_RootUrl redirects,
  *  and to ignore image redirects --Jcid)
@@ -1070,7 +1126,7 @@ typedef struct {
    DilloUrl *url;
 } Cache_savelink_t;
 
-/*
+/**
  * Save link from behind a timeout so that Cache_process_queue() can
  * get on with its work.
  */
@@ -1083,7 +1139,7 @@ static void Cache_savelink_cb(void *vdata)
    dFree(data);
 }
 
-/*
+/**
  * Let the client know that we're not following a redirection.
  */
 static void Cache_provide_redirection_blocked_page(CacheEntry_t *entry,
@@ -1094,7 +1150,7 @@ static void Cache_provide_redirection_blocked_page(CacheEntry_t *entry,
    a_Web_dispatch_by_type("text/html", clientWeb, &client->Callback,
                           &client->CbData);
    client->Buf = dStrconcat("<!doctype html><html><body>"
-                    "Dillo blocked a redirection attempt from <a href=\"",
+                    "Dillo blocked a redirection from <a href=\"",
                     URL_STR(entry->Url), "\">", URL_STR(entry->Url),
                     "</a> to <a href=\"", URL_STR(entry->Location), "\">",
                     URL_STR(entry->Location), "</a> based on your domainrc "
@@ -1104,7 +1160,7 @@ static void Cache_provide_redirection_blocked_page(CacheEntry_t *entry,
    dFree(client->Buf);
 }
 
-/*
+/**
  * Update cache clients for a single cache-entry
  * Tasks:
  *   - Set the client function (if not already set)
@@ -1112,7 +1168,7 @@ static void Cache_provide_redirection_blocked_page(CacheEntry_t *entry,
  *   - Remove clients when done
  *   - Call redirect handler
  *
- * Return: Cache entry, which may be NULL if it has been removed.
+ * @return Cache entry, which may be NULL if it has been removed.
  *
  * TODO: Implement CA_Abort Op in client callback
  */
@@ -1138,7 +1194,7 @@ static CacheEntry_t *Cache_process_queue(CacheEntry_t *entry)
       st = a_Misc_get_content_type_from_data(
               entry->Data->str, entry->Data->len, &Type);
       _MSG("Cache: detected Content-Type '%s'\n", Type);
-      if (st == 0 || entry->Flags & CA_GotData) {
+      if (st == 0 || !(entry->Flags & CA_InProgress)) {
          if (a_Misc_content_type_check(entry->TypeHdr, Type) < 0) {
             MSG_HTTP("Content-Type '%s' doesn't match the real data.\n",
                      entry->TypeHdr);
@@ -1176,7 +1232,7 @@ static CacheEntry_t *Cache_process_queue(CacheEntry_t *entry)
                Client_bw->redirect_level = 0;
             }
             if (entry->Flags & CA_HugeFile) {
-               a_UIcmd_set_msg(Client_bw,"Huge file! (%dMB)",
+               a_UIcmd_set_msg(Client_bw, "Huge file! (%d MB)",
                                entry->ExpectedSize / (1024*1024));
                AbortEntry = OfferDownload = TRUE;
             }
@@ -1243,7 +1299,7 @@ static CacheEntry_t *Cache_process_queue(CacheEntry_t *entry)
          }
 
          /* Remove client when done */
-         if (entry->Flags & CA_GotData) {
+         if (!(entry->Flags & CA_InProgress)) {
             /* Copy flags to a local var */
             int flags = ClientWeb->flags;
 
@@ -1253,12 +1309,17 @@ static CacheEntry_t *Cache_process_queue(CacheEntry_t *entry)
             }
             /* We finished sending data, let the client know */
             (Client->Callback)(CA_Close, Client);
-            if (ClientWeb->flags & WEB_RootUrl)
+            if (ClientWeb->flags & WEB_RootUrl) {
+               if (entry->Flags & CA_Aborted) {
+                  a_UIcmd_set_msg(Client_bw, "ERROR: Connection closed early, "
+                                             "read not complete.");
+               }
                a_UIcmd_set_page_prog(Client_bw, 0, 0);
+            }
             Cache_client_dequeue(Client);
             --i; /* Keep the index value in the next iteration */
 
-            /* within CA_GotData, we assert just one redirect call */
+            /* we assert just one redirect call */
             if (entry->Flags & CA_Redirect)
                Cache_redirect(entry, flags, Client_bw);
          }
@@ -1281,7 +1342,7 @@ static CacheEntry_t *Cache_process_queue(CacheEntry_t *entry)
          }
       }
       a_Url_free(url);
-   } else if (entry->Auth && (entry->Flags & CA_GotData)) {
+   } else if (entry->Auth && !(entry->Flags & CA_InProgress)) {
       Cache_auth_entry(entry, Client_bw);
    }
 
@@ -1295,12 +1356,13 @@ static CacheEntry_t *Cache_process_queue(CacheEntry_t *entry)
    return entry;
 }
 
-/*
+/**
  * Callback function for Cache_delayed_process_queue.
  */
-static void Cache_delayed_process_queue_callback()
+static void Cache_delayed_process_queue_callback(void *ptr)
 {
    CacheEntry_t *entry;
+   (void) ptr; /* Unused */
 
    while ((entry = (CacheEntry_t *)dList_nth_data(DelayedQueue, 0))) {
       Cache_ref_data(entry);
@@ -1313,7 +1375,7 @@ static void Cache_delayed_process_queue_callback()
    a_Timeout_remove();
 }
 
-/*
+/**
  * Set a call to Cache_process_queue from the main cycle.
  */
 static void Cache_delayed_process_queue(CacheEntry_t *entry)
@@ -1329,9 +1391,9 @@ static void Cache_delayed_process_queue(CacheEntry_t *entry)
    }
 }
 
-/*
+/**
  * Last Client for this entry?
- * Return: Client if true, NULL otherwise
+ * @return Client if true, NULL otherwise
  * (cache.c has only one call to a capi function. This avoids a second one)
  */
 CacheClient_t *a_Cache_client_get_if_unique(int Key)
@@ -1350,7 +1412,7 @@ CacheClient_t *a_Cache_client_get_if_unique(int Key)
    return (n == 1) ? Client : NULL;
 }
 
-/*
+/**
  * Remove a client from the client queue
  * TODO: notify the dicache and upper layers
  */
@@ -1380,7 +1442,7 @@ void a_Cache_stop_client(int Key)
 }
 
 
-/*
+/**
  * Memory deallocator (only called at exit time)
  */
 void a_Cache_freeall(void)
