@@ -3,6 +3,7 @@
  *
  * Copyright 2004 Sebastian Geerken <sgeerken@dillo.org>
  * Copyright 2008-2009 Johannes Hofmann <Johannes.Hofmann@gmx.de>
+ * Copyright 2024 Rodrigo Arias Mallo <rodarima@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,7 +28,8 @@
 
 using namespace dw::core::style;
 
-#define MSG_CSS(A, ...) MSG(A, __VA_ARGS__)
+//#define MSG_CSS(A, ...) _MSG(A, __VA_ARGS__)
+#define MSG_CSS(A, ...) do {} while(0)
 #define DEBUG_TOKEN_LEVEL   0
 #define DEBUG_PARSE_LEVEL   0
 #define DEBUG_CREATE_LEVEL  0
@@ -966,6 +968,24 @@ bool CssParser::parseValue(CssPropertyName prop,
             } else if (dStrAsciiCasecmp(tval, "ex") == 0) {
                lentype = CSS_LENGTH_TYPE_EX;
                nextToken();
+            } else if (dStrAsciiCasecmp(tval, "ch") == 0) {
+               lentype = CSS_LENGTH_TYPE_CH;
+               nextToken();
+            } else if (dStrAsciiCasecmp(tval, "rem") == 0) {
+               lentype = CSS_LENGTH_TYPE_REM;
+               nextToken();
+            } else if (dStrAsciiCasecmp(tval, "vw") == 0) {
+               lentype = CSS_LENGTH_TYPE_VW;
+               nextToken();
+            } else if (dStrAsciiCasecmp(tval, "vh") == 0) {
+               lentype = CSS_LENGTH_TYPE_VH;
+               nextToken();
+            } else if (dStrAsciiCasecmp(tval, "vmin") == 0) {
+               lentype = CSS_LENGTH_TYPE_VMIN;
+               nextToken();
+            } else if (dStrAsciiCasecmp(tval, "vmax") == 0) {
+               lentype = CSS_LENGTH_TYPE_VMAX;
+               nextToken();
             } else {
                ret = false;
             }
@@ -987,7 +1007,7 @@ bool CssParser::parseValue(CssPropertyName prop,
             (type == CSS_TYPE_LENGTH_PERCENTAGE_NUMBER || fval == 0.0))
             ret = true;
 
-         val->intVal = CSS_CREATE_LENGTH(fval, lentype);
+         val->lenVal = CSS_CREATE_LENGTH(fval, lentype);
       }
       break;
 
@@ -1090,7 +1110,7 @@ bool CssParser::parseValue(CssPropertyName prop,
       // possibilities are tested in parallel.
 
       bool h[2], v[2];
-      int pos[2];
+      CssLength pos[2];
       h[0] = v[0] = h[1] = v[1] = false;
 
       // First: collect values in pos[0] and pos[1], and determine whether
@@ -1133,10 +1153,10 @@ bool CssParser::parseValue(CssPropertyName prop,
                // We can assume <length> or <percentage> here ...
                CssPropertyValue valTmp;
                if (parseValue(prop, CSS_TYPE_LENGTH_PERCENTAGE, &valTmp)) {
-                  pos[i] = valTmp.intVal;
+                  pos[i] = valTmp.lenVal;
                   ret = true;
                } else if (parseValue(prop, CSS_TYPE_SIGNED_LENGTH, &valTmp)) {
-                  pos[i] = valTmp.intVal;
+                  pos[i] = valTmp.lenVal;
                   ret = true;
                } else
                   // ... but something may still fail.
