@@ -2,6 +2,7 @@
  * File: dicache.c
  *
  * Copyright 2000-2007 Jorge Arellano Cid <jcid@dillo.org>
+ * Copyright 2024 Rodrigo Arias Mallo <rodarima@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,18 +19,31 @@
 #include "web.hh"
 #include "dicache.h"
 #include "dpng.h"
+#include "dwebp.h"
 #include "dgif.h"
 #include "djpeg.h"
-
+#include "dsvg.h"
 
 enum {
-   DIC_Gif,
+   DIC_Gif = 0,
    DIC_Png,
-   DIC_Jpeg
+   DIC_Webp,
+   DIC_Jpeg,
+   DIC_Svg,
+   DIC_MAX
 };
 
+static const char *format_name[DIC_MAX] = {
+   [DIC_Gif]  = "gif",
+   [DIC_Png]  = "png",
+   [DIC_Webp] = "webp",
+   [DIC_Jpeg] = "jpeg",
+   [DIC_Svg]  = "svg"
+};
 
-/*
+static int disabled_formats[DIC_MAX] = { 0 };
+
+/**
  * List of DICacheEntry. May hold several versions of the same image,
  * although most of the time it holds just one.
  */
@@ -39,7 +53,7 @@ static uint_t dicache_size_total; /* invariant: dicache_size_total is
                                    * the sum of the image sizes (3*w*h)
                                    * of all the images in the dicache. */
 
-/*
+/**
  * Compare function for image entries
  */
 static int Dicache_entry_cmp(const void *v1, const void *v2)
@@ -56,16 +70,25 @@ static int Dicache_entry_cmp(const void *v1, const void *v2)
    return st;
 }
 
-/*
+/**
  * Initialize dicache data
  */
 void a_Dicache_init(void)
 {
    CachedIMGs = dList_new(256);
    dicache_size_total = 0;
+
+   if (prefs.ignore_image_formats) {
+      for (int i = 0; i < DIC_MAX; i++) {
+         if (dStriAsciiStr(prefs.ignore_image_formats, format_name[i])) {
+            disabled_formats[i] = 1;
+            _MSG("Image format %s disabled\n", format_name[i]);
+         }
+      }
+   }
 }
 
-/*
+/**
  * Create, and initialize a new, empty, dicache entry
  */
 static DICacheEntry *Dicache_entry_new(void)
@@ -93,7 +116,7 @@ static DICacheEntry *Dicache_entry_new(void)
    return entry;
 }
 
-/*
+/**
  * Add a new entry in the dicache
  * (a single URL may have several entries)
  */
@@ -117,9 +140,9 @@ static DICacheEntry *Dicache_add_entry(const DilloUrl *Url)
    return entry;
 }
 
-/*
+/**
  * Search a particular version of a URL in the Dicache.
- * Return value: a pointer to the entry if found; NULL otherwise.
+ * @return a pointer to the entry if found; NULL otherwise.
  *
  * Notes: DIC_Last means last version of the image.
  *        version zero is not allowed.
@@ -138,7 +161,7 @@ DICacheEntry *a_Dicache_get_entry(const DilloUrl *Url, int version)
    return entry;
 }
 
-/*
+/**
  * Actually free a dicache entry, given the URL and the version number.
  */
 static void Dicache_remove(const DilloUrl *Url, int version)
@@ -168,7 +191,7 @@ static void Dicache_remove(const DilloUrl *Url, int version)
    dFree(entry);
 }
 
-/*
+/**
  * Unrefs the counter of a dicache entry (it counts cache clients).
  * If there're no clients and no imgbuf, remove the entry.
  * Otherwise, let a_Dicache_cleanup() do the job later
@@ -188,7 +211,7 @@ void a_Dicache_unref(const DilloUrl *Url, int version)
    }
 }
 
-/*
+/**
  * Refs the counter of a dicache entry.
  */
 DICacheEntry* a_Dicache_ref(const DilloUrl *Url, int version)
@@ -201,7 +224,7 @@ DICacheEntry* a_Dicache_ref(const DilloUrl *Url, int version)
    return entry;
 }
 
-/*
+/**
  * Invalidate this entry. This is used for the reloading mechanism.
  * Can't erase current versions, but a_Dicache_get_entry(url, DIC_Last)
  * must return NULL.
@@ -216,7 +239,7 @@ void a_Dicache_invalidate_entry(const DilloUrl *Url)
 
 /* ------------------------------------------------------------------------- */
 
-/*
+/**
  * Set image's width, height & type
  * - 'width' and 'height' come from the image data.
  * - HTML width and height attrs are handled with setNonCssHint.
@@ -253,7 +276,7 @@ void a_Dicache_set_parms(DilloUrl *url, int version, DilloImage *Image,
    dicache_size_total += DicEntry->TotalSize;
 }
 
-/*
+/**
  * Implement the set_cmap method for the Image
  */
 void a_Dicache_set_cmap(DilloUrl *url, int version, int bg_color,
@@ -277,7 +300,7 @@ void a_Dicache_set_cmap(DilloUrl *url, int version, int bg_color,
    DicEntry->State = DIC_SetCmap;
 }
 
-/*
+/**
  * Reset for a new scan from a multiple-scan image.
  */
 void a_Dicache_new_scan(const DilloUrl *url, int version)
@@ -297,7 +320,7 @@ void a_Dicache_new_scan(const DilloUrl *url, int version)
    a_Imgbuf_new_scan(DicEntry->v_imgbuf);
 }
 
-/*
+/**
  * Implement the write method
  * (Write a scan line into the Dicache entry)
  * buf: row buffer
@@ -320,7 +343,7 @@ void a_Dicache_write(DilloUrl *url, int version, const uchar_t *buf, uint_t Y)
    DicEntry->State = DIC_Write;
 }
 
-/*
+/**
  * Implement the close method of the decoding process
  */
 void a_Dicache_close(DilloUrl *url, int version, CacheClient_t *Client)
@@ -350,16 +373,15 @@ void a_Dicache_close(DilloUrl *url, int version, CacheClient_t *Client)
 
 /* ------------------------------------------------------------------------- */
 
-/*
- * Generic MIME handler for GIF, JPEG and PNG.
+/**
+ * Generic MIME handler for GIF, JPEG, PNG and SVG.
  * Sets a_Dicache_callback as the cache-client,
  * and also sets the image decoder.
  *
- * Parameters:
- *   Type: MIME type
- *   Ptr:  points to a Web structure
- *   Call: Dillo calls this with more data/eod
- *   Data: Decoding data structure
+ * @param Type MIME type
+ * @param Ptr  points to a \ref DilloWeb structure
+ * @param Call Dillo calls this with more data/eod
+ * @param Data Decoding data structure
  */
 static void *Dicache_image(int ImgType, const char *MimeType, void *Ptr,
                            CA_Callback_t *Call, void **Data)
@@ -369,9 +391,14 @@ static void *Dicache_image(int ImgType, const char *MimeType, void *Ptr,
 
    dReturn_val_if_fail(MimeType && Ptr, NULL);
 
+   if (ImgType >= 0 && ImgType < DIC_MAX && disabled_formats[ImgType]) {
+      _MSG("Ignoring image format %s\n", format_name[ImgType]);
+      return NULL;
+   }
+
    if (!web->Image) {
       web->Image =
-         a_Image_new_with_dw(web->bw->render_layout, NULL, web->bgColor);
+         a_Image_new_with_dw(web->bw->render_layout, NULL, web->bgColor, 0);
       a_Image_ref(web->Image);
    }
 
@@ -388,10 +415,18 @@ static void *Dicache_image(int ImgType, const char *MimeType, void *Ptr,
          DicEntry->Decoder = (CA_Callback_t)a_Gif_callback;
          DicEntry->DecoderData =
             a_Gif_new(web->Image, DicEntry->url, DicEntry->version);
+      } else if (ImgType == DIC_Webp) {
+         DicEntry->Decoder = (CA_Callback_t)a_Webp_callback;
+         DicEntry->DecoderData =
+            a_Webp_new(web->Image, DicEntry->url, DicEntry->version);
       } else if (ImgType == DIC_Png) {
          DicEntry->Decoder = (CA_Callback_t)a_Png_callback;
          DicEntry->DecoderData =
             a_Png_new(web->Image, DicEntry->url, DicEntry->version);
+      } else if (ImgType == DIC_Svg) {
+         DicEntry->Decoder = (CA_Callback_t)a_Svg_callback;
+         DicEntry->DecoderData =
+            a_Svg_new(web->Image, DicEntry->url, DicEntry->version);
       }
    } else {
       /* Repeated image */
@@ -406,7 +441,7 @@ static void *Dicache_image(int ImgType, const char *MimeType, void *Ptr,
    return (a_Image_get_dw (web->Image));
 }
 
-/*
+/**
  * PNG wrapper for Dicache_image()
  */
 void *a_Dicache_png_image(const char *Type, void *Ptr, CA_Callback_t *Call,
@@ -415,7 +450,16 @@ void *a_Dicache_png_image(const char *Type, void *Ptr, CA_Callback_t *Call,
    return Dicache_image(DIC_Png, Type, Ptr, Call, Data);
 }
 
-/*
+/**
+ * WEBP wrapper for Dicache_image()
+ */
+void *a_Dicache_webp_image(const char *Type, void *Ptr, CA_Callback_t *Call,
+                          void **Data)
+{
+   return Dicache_image(DIC_Webp, Type, Ptr, Call, Data);
+}
+
+/**
  * GIF wrapper for Dicache_image()
  */
 void *a_Dicache_gif_image(const char *Type, void *Ptr, CA_Callback_t *Call,
@@ -424,7 +468,7 @@ void *a_Dicache_gif_image(const char *Type, void *Ptr, CA_Callback_t *Call,
    return Dicache_image(DIC_Gif, Type, Ptr, Call, Data);
 }
 
-/*
+/**
  * JPEG wrapper for Dicache_image()
  */
 void *a_Dicache_jpeg_image(const char *Type, void *Ptr, CA_Callback_t *Call,
@@ -433,7 +477,16 @@ void *a_Dicache_jpeg_image(const char *Type, void *Ptr, CA_Callback_t *Call,
    return Dicache_image(DIC_Jpeg, Type, Ptr, Call, Data);
 }
 
-/*
+/**
+ * SVG wrapper for Dicache_image()
+ */
+void *a_Dicache_svg_image(const char *Type, void *Ptr, CA_Callback_t *Call,
+                          void **Data)
+{
+   return Dicache_image(DIC_Svg, Type, Ptr, Call, Data);
+}
+
+/**
  * This function is a cache client; (but feeds its clients from dicache)
  */
 void a_Dicache_callback(int Op, CacheClient_t *Client)
@@ -501,7 +554,7 @@ void a_Dicache_callback(int Op, CacheClient_t *Client)
 
 /* ------------------------------------------------------------------------- */
 
-/*
+/**
  * Free the imgbuf (RGB data) of unused entries.
  */
 void a_Dicache_cleanup(void)
@@ -521,12 +574,12 @@ void a_Dicache_cleanup(void)
          --i; /* adjust counter */
       }
    }
-   MSG("a_Dicache_cleanup: length = %d\n", dList_length(CachedIMGs));
+   _MSG("a_Dicache_cleanup: length = %d\n", dList_length(CachedIMGs));
 }
 
 /* ------------------------------------------------------------------------- */
 
-/*
+/**
  * Deallocate memory used by dicache module
  * (Call this one at exit time, with no cache clients queued)
  */
