@@ -2,6 +2,7 @@
  * Dillo web browser
  *
  * Copyright 1999-2007 Jorge Arellano Cid <jcid@dillo.org>
+ * Copyright 2024 Rodrigo Arias Mallo <rodarima@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -46,6 +47,7 @@
 #include "bw.h"
 #include "misc.h"
 #include "history.h"
+#include "version.hh"
 
 #include "dns.h"
 #include "web.hh"
@@ -55,6 +57,7 @@
 #include "capi.h"
 #include "dicache.h"
 #include "cookies.h"
+#include "actions.h"
 #include "hsts.h"
 #include "domain.h"
 #include "auth.h"
@@ -65,6 +68,8 @@
 #include "dw/textblock.hh"
 #include "dw/table.hh"
 
+static volatile sig_atomic_t sig_reload = 0;
+
 /**
  * Command line options structure
  */
@@ -125,6 +130,8 @@ static void raw_sigchld2(int signum)
    pid_t pid;
    int status;
 
+   (void) signum; /* Unused */
+
    while (1) {
       pid = waitpid(-1, &status, WNOHANG);
       if (pid > 0) {
@@ -141,7 +148,11 @@ static void raw_sigchld2(int signum)
          break;
       }
    }
-   ++signum; /* compiler happiness */
+}
+
+static void handler_usr1(int signum)
+{
+   sig_reload = 1;
 }
 
 /**
@@ -160,6 +171,11 @@ static void est_sigchld(void)
       perror("sigaction");
       exit(1);
    }
+
+   if (signal(SIGUSR1, handler_usr1) == SIG_ERR) {
+      perror("signal failed");
+      exit(1);
+   }
 }
 
 //----------------------------------------------------------------------------
@@ -427,7 +443,7 @@ int main(int argc, char **argv)
          }
          break;
       case DILLO_CLI_VERSION:
-         puts("Dillo version " VERSION);
+         a_Version_print_info();
          return 0;
       case DILLO_CLI_HELP:
          printHelp(argv[0], Options);
@@ -474,6 +490,7 @@ int main(int argc, char **argv)
    a_Dicache_init();
    a_Bw_init();
    a_Cookies_init();
+   a_Actions_init();
    a_Hsts_init(Paths::getPrefsFP(PATHS_HSTS_PRELOAD));
    a_Auth_init();
    a_UIcmd_init();
@@ -579,7 +596,15 @@ int main(int argc, char **argv)
       }
    }
 
-   Fl::run();
+   /* Don't use, as it can be free()'d */
+   bw = NULL;
+
+   while (Fl::wait() > 0) {
+      if (sig_reload) {
+         sig_reload = 0;
+         a_UIcmd_reload_all_active();
+      }
+   }
 
    /*
     * Memory deallocating routines
