@@ -20,12 +20,12 @@ void a_Dns_resolve(const char *hostname, DnsCallback_t cb_func, void *cb_data);
 #  define DILLO_ADDR_MAX sizeof(struct in_addr)
 #endif
 
-typedef struct _DilloHost
-{
+typedef struct {
   int af;
   int alen;
   char data[DILLO_ADDR_MAX];
 } DilloHost;
+
 void a_Dns_dillohost_to_string(DilloHost *host, char *dst, size_t size);
 
 #ifdef __cplusplus
