@@ -48,6 +48,7 @@ typedef struct {
    char *no_proxy;
    DilloUrl *start_page;
    DilloUrl *home;
+   DilloUrl *new_tab_page;
    bool_t allow_white_bg;
    int32_t white_bg_replacement;
    int32_t bg_color;
@@ -72,9 +73,13 @@ typedef struct {
    bool_t adjust_table_min_width;
    bool_t focus_new_tab;
    double font_factor;
+   double zoom_factor;
    int32_t font_max_size;
    int32_t font_min_size;
    int32_t scroll_step;
+   int32_t scroll_page_overlap;
+   bool_t scrollbar_on_left;
+   bool_t scrollbar_page_mode;
    bool_t show_back;
    bool_t show_forw;
    bool_t show_home;
@@ -92,6 +97,7 @@ typedef struct {
    bool_t show_quit_dialog;
    bool_t fullwindow_start;
    bool_t load_images;
+   char *ignore_image_formats;
    bool_t load_background_images;
    bool_t load_stylesheets;
    bool_t parse_embedded_css;
@@ -118,6 +124,7 @@ typedef struct {
    int penalty_hyphen, penalty_hyphen_2;
    int penalty_em_dash_left, penalty_em_dash_right, penalty_em_dash_right_2;
    int stretchability_factor;
+   Dlist *link_actions;
 } DilloPrefs;
 
 /** Global Data */
