@@ -60,7 +60,8 @@ void StyleImageDeletionReceiver::deleted (lout::signal::ObservedObject *object)
 // ----------------------------------------------------------------------
 
 StyleEngine::StyleEngine (dw::core::Layout *layout,
-                          const DilloUrl *pageUrl, const DilloUrl *baseUrl) {
+                          const DilloUrl *pageUrl, const DilloUrl *baseUrl,
+                          float zoom) {
    StyleAttrs style_attrs;
    FontAttrs font_attrs;
 
@@ -73,13 +74,14 @@ StyleEngine::StyleEngine (dw::core::Layout *layout,
    this->baseUrl = baseUrl ? a_Url_dup(baseUrl) : NULL;
    importDepth = 0;
    dpmm = layout->dpiX () / 25.4; /* assume dpiX == dpiY */
+   this->zoom = zoom;
 
    stackPush ();
    Node *n = stack->getLastRef ();
 
    /* Create a dummy font, attribute, and tag for the bottom of the stack. */
    font_attrs.name = prefs.font_sans_serif;
-   font_attrs.size = roundInt(14 * prefs.font_factor);
+   font_attrs.size = roundInt(14 * prefs.font_factor * zoom);
    if (font_attrs.size < prefs.font_min_size)
       font_attrs.size = prefs.font_min_size;
    if (font_attrs.size > prefs.font_max_size)
@@ -410,37 +412,37 @@ void StyleEngine::apply (int i, StyleAttrs *attrs, CssPropertyList *props,
             if (p->type == CSS_TYPE_ENUM) {
                switch (p->value.intVal) {
                   case CSS_FONT_SIZE_XX_SMALL:
-                     fontAttrs.size = roundInt(8.1 * prefs.font_factor);
+                     fontAttrs.size = roundInt(8.1 * prefs.font_factor * zoom);
                      break;
                   case CSS_FONT_SIZE_X_SMALL:
-                     fontAttrs.size = roundInt(9.7 * prefs.font_factor);
+                     fontAttrs.size = roundInt(9.7 * prefs.font_factor * zoom);
                      break;
                   case CSS_FONT_SIZE_SMALL:
-                     fontAttrs.size = roundInt(11.7 * prefs.font_factor);
+                     fontAttrs.size = roundInt(11.7 * prefs.font_factor * zoom);
                      break;
                   case CSS_FONT_SIZE_MEDIUM:
-                     fontAttrs.size = roundInt(14.0 * prefs.font_factor);
+                     fontAttrs.size = roundInt(14.0 * prefs.font_factor * zoom);
                      break;
                   case CSS_FONT_SIZE_LARGE:
-                     fontAttrs.size = roundInt(16.8 * prefs.font_factor);
+                     fontAttrs.size = roundInt(16.8 * prefs.font_factor * zoom);
                      break;
                   case CSS_FONT_SIZE_X_LARGE:
-                     fontAttrs.size = roundInt(20.2 * prefs.font_factor);
+                     fontAttrs.size = roundInt(20.2 * prefs.font_factor * zoom);
                      break;
                   case CSS_FONT_SIZE_XX_LARGE:
-                     fontAttrs.size = roundInt(24.2 * prefs.font_factor);
+                     fontAttrs.size = roundInt(24.2 * prefs.font_factor * zoom);
                      break;
                   case CSS_FONT_SIZE_SMALLER:
-                     fontAttrs.size = roundInt(fontAttrs.size * 0.83);
+                     fontAttrs.size = roundInt(fontAttrs.size * 0.83 * zoom);
                      break;
                   case CSS_FONT_SIZE_LARGER:
-                     fontAttrs.size = roundInt(fontAttrs.size * 1.2);
+                     fontAttrs.size = roundInt(fontAttrs.size * 1.2 * zoom);
                      break;
                   default:
                      assert(false); // invalid font-size enum
                }
             } else {
-               computeValue (&fontAttrs.size, p->value.intVal, parentFont,
+               computeValue (&fontAttrs.size, p->value.lenVal, parentFont,
                   parentFont->size);
             }
 
@@ -492,7 +494,7 @@ void StyleEngine::apply (int i, StyleAttrs *attrs, CssPropertyList *props,
                   fontAttrs.letterSpacing = 0;
                }
             } else {
-               computeValue (&fontAttrs.letterSpacing, p->value.intVal,
+               computeValue (&fontAttrs.letterSpacing, p->value.lenVal,
                   parentFont, parentFont->size);
             }
 
@@ -587,11 +589,11 @@ void StyleEngine::apply (int i, StyleAttrs *attrs, CssPropertyList *props,
             computeBorderWidth (&attrs->borderWidth.top, p, attrs->font);
             break;
          case CSS_PROPERTY_BORDER_SPACING:
-            computeValue (&attrs->hBorderSpacing, p->value.intVal,attrs->font);
-            computeValue (&attrs->vBorderSpacing, p->value.intVal,attrs->font);
+            computeValue (&attrs->hBorderSpacing, p->value.lenVal,attrs->font);
+            computeValue (&attrs->vBorderSpacing, p->value.lenVal,attrs->font);
             break;
          case CSS_PROPERTY_BOTTOM:
-            computeLength (&attrs->bottom, p->value.intVal, attrs->font);
+            computeLength (&attrs->bottom, p->value.lenVal, attrs->font);
             break;
          case CSS_PROPERTY_CLEAR:
             attrs->clear = (ClearType) p->value.intVal;
@@ -611,16 +613,16 @@ void StyleEngine::apply (int i, StyleAttrs *attrs, CssPropertyList *props,
             attrs->vloat = (FloatType) p->value.intVal;
             break;
          case CSS_PROPERTY_LEFT:
-            computeLength (&attrs->left, p->value.intVal, attrs->font);
+            computeLength (&attrs->left, p->value.lenVal, attrs->font);
             break;
          case CSS_PROPERTY_LINE_HEIGHT:
             if (p->type == CSS_TYPE_ENUM) { //only valid enum value is "normal"
                attrs->lineHeight = dw::core::style::LENGTH_AUTO;
             } else if (p->type == CSS_TYPE_LENGTH_PERCENTAGE_NUMBER) {
-               if (CSS_LENGTH_TYPE (p->value.intVal) == CSS_LENGTH_TYPE_NONE) {
+               if (CSS_LENGTH_TYPE (p->value.lenVal) == CSS_LENGTH_TYPE_NONE) {
                   attrs->lineHeight =
-                     createPerLength(CSS_LENGTH_VALUE(p->value.intVal));
-               } else if (computeValue (&lineHeight, p->value.intVal,
+                     createPerLength(CSS_LENGTH_VALUE(p->value.lenVal));
+               } else if (computeValue (&lineHeight, p->value.lenVal,
                                         attrs->font, attrs->font->size)) {
                   attrs->lineHeight = createAbsLength(lineHeight);
                }
@@ -633,22 +635,22 @@ void StyleEngine::apply (int i, StyleAttrs *attrs, CssPropertyList *props,
             attrs->listStyleType = (ListStyleType) p->value.intVal;
             break;
          case CSS_PROPERTY_MARGIN_BOTTOM:
-            computeValue (&attrs->margin.bottom, p->value.intVal, attrs->font);
+            computeValue (&attrs->margin.bottom, p->value.lenVal, attrs->font);
             if (attrs->margin.bottom < 0) // \todo fix negative margins in dw/*
                attrs->margin.bottom = 0;
             break;
          case CSS_PROPERTY_MARGIN_LEFT:
-            computeValue (&attrs->margin.left, p->value.intVal, attrs->font);
+            computeValue (&attrs->margin.left, p->value.lenVal, attrs->font);
             if (attrs->margin.left < 0) // \todo fix negative margins in dw/*
                attrs->margin.left = 0;
             break;
          case CSS_PROPERTY_MARGIN_RIGHT:
-            computeValue (&attrs->margin.right, p->value.intVal, attrs->font);
+            computeValue (&attrs->margin.right, p->value.lenVal, attrs->font);
             if (attrs->margin.right < 0) // \todo fix negative margins in dw/*
                attrs->margin.right = 0;
             break;
          case CSS_PROPERTY_MARGIN_TOP:
-            computeValue (&attrs->margin.top, p->value.intVal, attrs->font);
+            computeValue (&attrs->margin.top, p->value.lenVal, attrs->font);
             if (attrs->margin.top < 0) // \todo fix negative margins in dw/*
                attrs->margin.top = 0;
             break;
@@ -656,22 +658,22 @@ void StyleEngine::apply (int i, StyleAttrs *attrs, CssPropertyList *props,
             attrs->overflow = (Overflow) p->value.intVal;
             break;
          case CSS_PROPERTY_PADDING_TOP:
-            computeValue (&attrs->padding.top, p->value.intVal, attrs->font);
+            computeValue (&attrs->padding.top, p->value.lenVal, attrs->font);
             break;
          case CSS_PROPERTY_PADDING_BOTTOM:
-            computeValue (&attrs->padding.bottom, p->value.intVal,attrs->font);
+            computeValue (&attrs->padding.bottom, p->value.lenVal,attrs->font);
             break;
          case CSS_PROPERTY_PADDING_LEFT:
-            computeValue (&attrs->padding.left, p->value.intVal, attrs->font);
+            computeValue (&attrs->padding.left, p->value.lenVal, attrs->font);
             break;
          case CSS_PROPERTY_PADDING_RIGHT:
-            computeValue (&attrs->padding.right, p->value.intVal, attrs->font);
+            computeValue (&attrs->padding.right, p->value.lenVal, attrs->font);
             break;
          case CSS_PROPERTY_POSITION:
             attrs->position = (Position) p->value.intVal;
             break;
          case CSS_PROPERTY_RIGHT:
-            computeLength (&attrs->right, p->value.intVal, attrs->font);
+            computeLength (&attrs->right, p->value.lenVal, attrs->font);
             break;
          case CSS_PROPERTY_TEXT_ALIGN:
             attrs->textAlign = (TextAlignType) p->value.intVal;
@@ -680,13 +682,13 @@ void StyleEngine::apply (int i, StyleAttrs *attrs, CssPropertyList *props,
             attrs->textDecoration |= p->value.intVal;
             break;
          case CSS_PROPERTY_TEXT_INDENT:
-            computeLength (&attrs->textIndent, p->value.intVal, attrs->font);
+            computeLength (&attrs->textIndent, p->value.lenVal, attrs->font);
             break;
          case CSS_PROPERTY_TEXT_TRANSFORM:
             attrs->textTransform = (TextTransform) p->value.intVal;
             break;
          case CSS_PROPERTY_TOP:
-            computeLength (&attrs->top, p->value.intVal, attrs->font);
+            computeLength (&attrs->top, p->value.lenVal, attrs->font);
             break;
          case CSS_PROPERTY_VERTICAL_ALIGN:
             attrs->valign = (VAlignType) p->value.intVal;
@@ -695,10 +697,10 @@ void StyleEngine::apply (int i, StyleAttrs *attrs, CssPropertyList *props,
             attrs->whiteSpace = (WhiteSpace) p->value.intVal;
             break;
          case CSS_PROPERTY_WIDTH:
-            computeLength (&attrs->width, p->value.intVal, attrs->font);
+            computeLength (&attrs->width, p->value.lenVal, attrs->font);
             break;
          case CSS_PROPERTY_HEIGHT:
-            computeLength (&attrs->height, p->value.intVal, attrs->font);
+            computeLength (&attrs->height, p->value.lenVal, attrs->font);
             break;
          case CSS_PROPERTY_WORD_SPACING:
             if (p->type == CSS_TYPE_ENUM) {
@@ -706,7 +708,7 @@ void StyleEngine::apply (int i, StyleAttrs *attrs, CssPropertyList *props,
                   attrs->wordSpacing = 0;
                }
             } else {
-               computeValue(&attrs->wordSpacing, p->value.intVal, attrs->font);
+               computeValue(&attrs->wordSpacing, p->value.lenVal, attrs->font);
             }
 
             /* Limit to reasonable values to avoid overflows */
@@ -716,16 +718,16 @@ void StyleEngine::apply (int i, StyleAttrs *attrs, CssPropertyList *props,
                attrs->wordSpacing = -1000;
             break;
          case CSS_PROPERTY_MIN_WIDTH:
-            computeLength (&attrs->minWidth, p->value.intVal, attrs->font);
+            computeLength (&attrs->minWidth, p->value.lenVal, attrs->font);
             break;
          case CSS_PROPERTY_MAX_WIDTH:
-            computeLength (&attrs->maxWidth, p->value.intVal, attrs->font);
+            computeLength (&attrs->maxWidth, p->value.lenVal, attrs->font);
             break;
          case CSS_PROPERTY_MIN_HEIGHT:
-            computeLength (&attrs->minHeight, p->value.intVal, attrs->font);
+            computeLength (&attrs->minHeight, p->value.lenVal, attrs->font);
             break;
          case CSS_PROPERTY_MAX_HEIGHT:
-            computeLength (&attrs->maxHeight, p->value.intVal, attrs->font);
+            computeLength (&attrs->maxHeight, p->value.lenVal, attrs->font);
             break;
          case CSS_PROPERTY_Z_INDEX:
             if (p->type == CSS_LENGTH_TYPE_AUTO)
@@ -763,7 +765,7 @@ void StyleEngine::apply (int i, StyleAttrs *attrs, CssPropertyList *props,
          a_Image_new(layout,
             (void*)attrs->backgroundImage
             ->getMainImgRenderer(),
-            0xffffff);
+            0xffffff, 0x000000);
 
       // we use the pageUrl as requester to prevent cross
       // domain requests as specified in domainrc
@@ -789,17 +791,45 @@ void StyleEngine::apply (int i, StyleAttrs *attrs, CssPropertyList *props,
 bool StyleEngine::computeValue (int *dest, CssLength value, Font *font) {
    switch (CSS_LENGTH_TYPE (value)) {
       case CSS_LENGTH_TYPE_PX:
-         *dest = (int) CSS_LENGTH_VALUE (value);
+         *dest = roundInt (CSS_LENGTH_VALUE (value) * zoom);
          return true;
       case CSS_LENGTH_TYPE_MM:
-         *dest = roundInt (CSS_LENGTH_VALUE (value) * dpmm);
+         *dest = roundInt (CSS_LENGTH_VALUE (value) * dpmm * zoom);
          return true;
       case CSS_LENGTH_TYPE_EM:
+         /* Doesn't need zoom as it is already applied to font->size */
          *dest = roundInt (CSS_LENGTH_VALUE (value) * font->size);
          return true;
       case CSS_LENGTH_TYPE_EX:
+         /* Doesn't need zoom as it is already applied to font->xHeight */
          *dest = roundInt (CSS_LENGTH_VALUE(value) * font->xHeight);
          return true;
+      case CSS_LENGTH_TYPE_CH:
+         *dest = roundInt (CSS_LENGTH_VALUE(value) * font->zeroWidth);
+         return true;
+      case CSS_LENGTH_TYPE_REM:
+         if (stack->size() < 2) {
+            *dest = 0;
+         } else {
+            dw::core::style::Style *root_style = stack->getRef (1)->style;
+            if (root_style)
+               *dest = roundInt (CSS_LENGTH_VALUE(value) * root_style->font->size);
+            else
+               *dest = 0;
+         }
+         return true;
+      case CSS_LENGTH_TYPE_VW:
+         *dest = roundInt (CSS_LENGTH_VALUE(value) * layout->getWidthViewport() / 100);
+         return true;
+      case CSS_LENGTH_TYPE_VH:
+         *dest = roundInt (CSS_LENGTH_VALUE(value) * layout->getHeightViewport() / 100);
+         return true;
+      case CSS_LENGTH_TYPE_VMIN:
+         *dest = roundInt (CSS_LENGTH_VALUE(value) * MIN(layout->getWidthViewport(), layout->getHeightViewport()) / 100);
+         return true;
+      case CSS_LENGTH_TYPE_VMAX:
+         *dest = roundInt (CSS_LENGTH_VALUE(value) * MAX(layout->getWidthViewport(), layout->getHeightViewport()) / 100);
+         return true;
       case CSS_LENGTH_TYPE_NONE:
          // length values other than 0 without unit are only allowed
          // in special cases (line-height) and have to be handled
@@ -858,7 +888,7 @@ void StyleEngine::computeBorderWidth (int *dest, CssProperty *p,
             assert(false);
       }
    } else {
-      computeValue (dest, p->value.intVal, font);
+      computeValue (dest, p->value.lenVal, font);
    }
 }
 
@@ -1001,7 +1031,6 @@ void StyleEngine::init () {
       " h1, h2, h3, h4, h5, h6, header, main, nav, ol, p, pre, section, ul"
       " {display: block}"
       "i, em, cite, address, var {font-style: italic}"
-      ":link img, :visited img {border: 1px solid}"
       "frameset, ul, ol, dir {margin-left: 40px}"
       /* WORKAROUND: It should be margin: 1em 0
        * but as we don't collapse these margins yet, it
