@@ -2,6 +2,7 @@
  * File: table.cc
  *
  * Copyright 2008 Jorge Arellano Cid <jcid@dillo.org>
+ * Copyright 2024 Rodrigo Arias Mallo <rodarima@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -9,12 +10,18 @@
  * (at your option) any later version.
  */
 
+/**
+ * @file
+ * Table parsing functions
+ */
+
 #include "table.hh"
 #include "html_common.hh"
 
 #include "dw/style.hh"
 #include "dw/textblock.hh"
 #include "dw/table.hh"
+#include "dw/simpletablecell.hh"
 
 #include "prefs.h"
 #include "msg.h"
@@ -243,6 +250,31 @@ void Html_tag_content_td(DilloHtml *html, const char *tag, int tagsize)
    Html_tag_content_table_cell (html, tag, tagsize);
 }
 
+/*
+ * <tbody>
+ */
+void Html_tag_open_tbody(DilloHtml *html, const char *tag, int tagsize)
+{
+}
+
+void Html_tag_content_tbody(DilloHtml *html, const char *tag, int tagsize)
+{
+}
+
+/*
+ * <tfoot>
+ */
+void Html_tag_open_tfoot(DilloHtml *html, const char *tag, int tagsize)
+{
+}
+
+/*
+ * <thead>
+ */
+void Html_tag_open_thead(DilloHtml *html, const char *tag, int tagsize)
+{
+}
+
 /*
  * <TH>
  */
@@ -261,7 +293,7 @@ void Html_tag_content_th(DilloHtml *html, const char *tag, int tagsize)
  * Utilities
  */
 
-/*
+/**
  * The table border model is stored in the table's stack item
  */
 static int Html_table_get_border_model(DilloHtml *html)
@@ -276,7 +308,7 @@ static int Html_table_get_border_model(DilloHtml *html)
    return html->stack->getRef(s_idx)->table_border_mode;
 }
 
-/*
+/**
  * Set current table's border model
  */
 static void Html_table_set_border_model(DilloHtml *html,
@@ -335,7 +367,7 @@ static void Html_set_collapsing_border_model(DilloHtml *html, Widget *col_tb)
    }
 }
 
-/*
+/**
  * Adjust style for separate border model.
  * (Dw uses this model internally).
  */
@@ -407,7 +439,7 @@ static void Html_tag_open_table_cell(DilloHtml *html,
             BUG_MSG("<t%c> bgcolor attribute is obsolete.",
                (tagsize >=3 && (D_ASCII_TOLOWER(tag[2]) == 'd')) ? 'd' : 'h');
       }
-
+      break;
    default:
       /* compiler happiness */
       break;
@@ -431,7 +463,7 @@ static void Html_tag_content_table_cell(DilloHtml *html,
       BUG_MSG("<t%c> outside <tr>.",
               (tagsize >=3 && (D_ASCII_TOLOWER(tag[2]) == 'd')) ? 'd' : 'h');
       /* a_Dw_table_add_cell takes care that dillo does not crash. */
-      /* continues */
+      /* fallthrough */
    case DILLO_HTML_TABLE_MODE_TR:
    case DILLO_HTML_TABLE_MODE_TD:
       if ((attrbuf = a_Html_get_attr(html, tag, tagsize, "colspan"))) {
@@ -445,11 +477,11 @@ static void Html_tag_content_table_cell(DilloHtml *html,
          rowspan = MAX(1, strtol (attrbuf, NULL, 10));
       if (html->style ()->textAlign
           == TEXT_ALIGN_STRING)
-         col_tb = new dw::TableCell (
+         col_tb = new AlignedTableCell (
                      ((dw::Table*)S_TOP(html)->table)->getCellRef (),
                      prefs.limit_text_width);
       else
-         col_tb = new Textblock (prefs.limit_text_width);
+         col_tb = new SimpleTableCell (prefs.limit_text_width);
 
       if (html->style()->borderCollapse == BORDER_MODEL_COLLAPSE){
          Html_set_collapsing_border_model(html, col_tb);
