@@ -148,7 +148,7 @@ static int Dpi_check_dpid_ids()
    /* socket connection test */
    memset(&sin, 0, sizeof(sin));
    sin.sin_family = AF_INET;
-   sin.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
+   sin.sin_addr.s_addr = inet_addr("127.0.0.1");
 
    if (Dpi_read_comm_keys(&dpid_port) != -1) {
       sin.sin_port = htons(dpid_port);
@@ -282,7 +282,7 @@ static int Dpi_blocking_start_dpid(void)
    /* test the dpid, and wait a bit for it to start if necessary */
    while ((cst = Dpi_check_dpid(n_tries)) == 1) {
       MSG("Dpi_blocking_start_dpid: try %d\n", ++try);
-      usleep(250000); /* 1/4 sec */
+      dUsleep(250000U); /* 1/4 sec */
    }
    return cst;
 }
@@ -317,7 +317,7 @@ static int Dpi_get_server_port(const char *server_name)
       sin_sz = sizeof(sin);
       memset(&sin, 0, sizeof(sin));
       sin.sin_family = AF_INET;
-      sin.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
+      sin.sin_addr.s_addr = inet_addr("127.0.0.1");
       sin.sin_port = htons(dpid_port);
       if ((sock_fd = Dpi_make_socket_fd()) == -1 ||
           connect(sock_fd, (struct sockaddr *)&sin, sin_sz) == -1) {
@@ -385,7 +385,7 @@ static int Dpi_connect_socket(const char *server_name)
    /* connect with this server's socket */
    memset(&sin, 0, sizeof(sin));
    sin.sin_family = AF_INET;
-   sin.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
+   sin.sin_addr.s_addr = inet_addr("127.0.0.1");
    sin.sin_port = htons(dpi_port);
 
    if ((sock_fd = Dpi_make_socket_fd()) == -1) {
